#include "aBPn_fn.h"
#include "aa_empirical.h"
void seq_genc(int smodel, int ntaxa, double *utreec, int nsite, int *seq, 
	      int nrate, double alpha, int iseed, double *fr, double *Qk)
{
  /* IN:
   * smodel - 0:JC, 1:F81, 2:F84, 3:GTR, 4:PAM, 5:JTT, 6:WAG, 7:mtREV24, 8:HKY,
   *          9:LG
   * nrate, alpha - if alpha > 0, gamma model generation. If nrate>0, nrate rates
   * iseed - used to set seed
   * fr - frequencies if different than model frequencies
   *      if *fr < 0 for aa models, model-based frequencies used
   * Qk - rate matrix for GTR, ttratio, K or Kappa for F84, HKY, < 0 => ttratio; 
   * not used otherwise.
   *
   * OUT:
   * seq
   * infile to tmp_seq_genc.infile 
   * stderr to tmp_seq_genc.out
   * tree in tmp_seq_genc.intree
   *    all removed afterwards
   */
  FILE *fp;
  char (*names)[11],tstring[10000],cmdline[1000],tmpstr[1000],*seqc,*oseqc;
  char smodels[10][8]={"F84","F84","F84","GTR","PAM","JTT","WAG","MTREV","HKY",
		       "GENERAL"};
  int i,j,k,nchar=4,smodell;
  double fro[20],W[400],Lam[20],R[400];

  seqc = (char *) malloc((size_t) nsite*ntaxa*sizeof(char));
 
  /* create seq-gen command and run */
  if (smodel == 4 || smodel == 5 || smodel == 6 || smodel == 7 || smodel == 9) 
    nchar = 20;
  names = (char (*)[11]) malloc((size_t) ntaxa*sizeof(*names));
  for(i = 0; i < ntaxa; i++) sprintf(names[i],"%i",i);
  /* pr_utreec(stdout,ntaxa,utreec); */
  ut2n(ntaxa,utreec,names,tstring);
/*   printf("%s\n",tstring); */
  fp=fopen("tmp_seq_genc.intree","w");
  fprintf(fp,"%s\n",tstring);
  fclose(fp);
  smodell=smodel;
  if(smodel==3 && nchar==20) smodell=9;
  sprintf(cmdline,"seq-gen -m%s -z%i -l%i",smodels[smodell], iseed, nsite);
  if (alpha > 0){
    sprintf(tmpstr," -a%f",alpha);
    if(nrate > 0) sprintf(tmpstr," -g%i",nrate);
    strcat(cmdline,tmpstr);
  }
  /* printf("%s\n",cmdline); */
  if (smodel == 0 || smodel == 1 || smodel == 2 || smodel == 8){
    if (smodel == 0 || smodel == 1) sprintf(tmpstr," -t%f",K2tt(0,fr));
    if (smodel == 2){ 
      if (*Qk < 0)
	sprintf(tmpstr," -t%f",-(*Qk));
      else
	sprintf(tmpstr," -t%f",K2tt(*Qk,fr));
    }
    if (smodel == 8){ 
      if (*Qk < 0)
	sprintf(tmpstr," -t%f",-(*Qk));
      else
	sprintf(tmpstr," -t%f",kappa2tt(*Qk,fr));
    }
    strcat(cmdline,tmpstr);
  }
  if (*fr > 0){
    strcat(cmdline," -f");
    for(i = 0; i < nchar; i++){
      if(i==0) 
	sprintf(tmpstr,"%f",fr[i]);
      else
	sprintf(tmpstr,",%f",fr[i]);
      strcat(cmdline,tmpstr);
    }
  }
  if(smodel == 3){
    strcat(cmdline," -r");
    for(i = 0; i < nchar; i++)
      for(j = i+1; j < nchar; j++){
	if(i==0&&j==0)
	  sprintf(tmpstr,"%f",Qk[i+j*nchar]/fr[j]);
	else
	  sprintf(tmpstr,",%f",Qk[i+j*nchar]/fr[j]);
	strcat(cmdline,tmpstr);
      }
  }
  
  if(smodel==9){/* LG model not directly avail. use GTR */
    makeprotfreqs(fro,lgprobmat,lgeigmat);
    if (*fr > 0){
      memcpy(W,lgprobmat,(size_t) nchar*nchar*sizeof(double));
      memcpy(Lam,lgeigmat,(size_t) nchar*sizeof(double));
      chQ_freq(nchar,W,Lam,fr,fro);
      WLamfr2exchange(nchar,W,Lam,fr,R);
    }
    if (*fr < 0){
      strcat(cmdline," -f");
      for(i = 1; i < nchar; i++){
	if(i==0)
	  sprintf(tmpstr,"%f",fro[i]);
	else
	  sprintf(tmpstr,",%f",fro[i]);
	strcat(cmdline,tmpstr);
      }
      WLamfr2exchange(nchar,lgprobmat,lgeigmat,fro,R);
    }
    strcat(cmdline," -r");
    for(i = 0; i < nchar; i++)
      for(j = i+1; j < nchar; j++){
	if(i==0&&j==0)
	  sprintf(tmpstr,"%f",R[i+j*nchar]);
	else
	  sprintf(tmpstr,",%f",R[i+j*nchar]);
	strcat(cmdline,tmpstr);
      }
  }

  strcat(cmdline," < tmp_seq_genc.intree > tmp_seq_genc.infile 2> tmp_seq_genc.outfile");
  /* printf("%s\n",cmdline); */
  system(cmdline);
  
  /* read in the sequence */
  fp=fopen("tmp_seq_genc.infile","r");
  fscanf(fp,"%i %i",&ntaxa,&nsite);
  rinterleavef_nolim(fp,&ntaxa,&nsite,names,seqc);
  fclose(fp);
  remove("tmp_seq_genc.infile");
  remove("tmp_seq_genc.intree");
  remove("tmp_seq_genc.outfile");

  /* sort so that integer names ordered low to high */
  oseqc = (char *) malloc(nsite*ntaxa*sizeof(char));
  for(j = 0; j < ntaxa; j++)
    for(k = 0; k < ntaxa; k++)
      if (atoi(names[k]) == j)
	for(i = 0; i < nsite; i++) oseqc[i+j*nsite]=seqc[i+k*nsite];
  if (nchar == 20)
    for(i = 0; i < nsite*ntaxa; i++) seq[i]=l2ip(oseqc[i]);
  if (nchar == 4)
    for(i = 0; i < nsite*ntaxa; i++) seq[i]=l2i(oseqc[i]);
  free(names); free(seqc); free(oseqc);
  return;
}
void aBPf_fast(int nBP, double *BP, int ntaxa, double *utreec,
	       int ns, int *s,
	       int nchar, int smodel, double *fr, double *Qk,
	       int nrate, double *rate, double *wt,
	       int tinfo, int nsite, int *seq,
	       int nsim, int condl, int iseed, int ismle, double *aBP){
  int i,j,nsimb,is;
  double *infom,Sc[9],Ic[3],*sBP,rho[3];
  
  /* information matrix calculation */
  infom = (double *) malloc((size_t) ns*(2*ntaxa-1)*(2*ntaxa-1)*sizeof(double));
/*   printf("ntaxa:%i ns:%i\n",ntaxa,ns); */
/*   for(i = 0; i < ns; i++) printf("%i ",s[i]); */
/*   printf("nchar:%i smodel:%i nrate:%i tinfo:%i nsite:%i\n", */
/* 	 nchar,smodel,nrate,tinfo,nsite); */
  infomatrixz(ntaxa,utreec,ns,s,nchar,smodel,fr,Qk,nrate,rate,wt,tinfo,nsite,
	      seq,infom,iseed);
  /* simulated bootstrap support values */
  sBP = (double *) malloc((size_t) nsim*sizeof(double));
  srand48(iseed);
  /* srand(iseed); /\* windows *\/ */
  for(is = 0; is < ns; is++){
    /* correlation matrix calculation */
/*       wmat(stdout,2*ntaxa-1,2*ntaxa-1,infom); */
    /*   wmate16(stdout,2*ntaxa-1,2*ntaxa-1,infom); */
    corscorecor(ntaxa,&infom[is*(2*ntaxa-1)*(2*ntaxa-1)],Sc,Ic);
    /*   wmat(stdout,3,3,Sc); */
    rho[0]=Sc[1]; rho[1]=Sc[2]; rho[2]=Sc[5];
/*     wrow(stdout,3,rho,1); */
    
    nsimb=bpnf(nsim, rho, sBP, condl, ismle);
    if(nsimb==0){
      printf("Error: no cases with split being MLE");
      exit(1);
    }
    for(j = 0; j < nBP; j++){
      aBP[is+j*ns] = 0.0;
      for(i = 0; i < nsimb; i++)
	if(sBP[i] <= BP[is+j*ns]) aBP[is+j*ns] += 1.0;
      aBP[is+j*ns] /= nsimb;
    }
  }
  free(infom); free(sBP);
}
void aBPuf_fast(double *BP, int ntaxa, double *utreec,
		int nchar, int smodel, double *fr, double *Qk,
		int nrate, double *rate, double *wt,
		int tinfo, int nsite, int *seq,
		int nsim, int condl, int iseed, int ismle, double *aBP){
  double *aBPn,*BPn,tol=1.0e-6;;
  int *s,ns=0,rt,i;
  aBPn = (double *) malloc((size_t) (ntaxa-3)*sizeof(double));
  BPn = (double *) malloc((size_t) (ntaxa-3)*sizeof(double));
  s = (int *) malloc((size_t) (ntaxa-3)*sizeof(int));
  for(rt = 0; rt < ntaxa-3; rt++){
    if (BP[rt] >= 1.0-tol)
      aBP[rt] = 1.0;
    else{
      s[ns] = rt+ntaxa;
      aBPn[ns] = aBP[rt];
      BPn[ns] = BP[rt];
      ns++;
    }
  }
  if(ns > 0)
    aBPf_fast(1,BPn,ntaxa,utreec,ns,s,nchar,smodel,fr,Qk,nrate,rate,wt,
	      tinfo,nsite,seq,nsim,condl,iseed,ismle,aBPn);
  for(i = 0; i < ns; i++)
    for(rt = 0; rt < ntaxa-3; rt++)
      if(s[i]==rt+ntaxa) aBP[rt] = aBPn[i];
  free(aBPn); free(BPn); free(s);
}
char *aBPlf(FILE *treefile, FILE *seqfile, int ntaxa, 
	    int nchar, int smodel, double *fr, double *Qk, 
	    int nrate, double *rate, double *wt,
	    int tinfo, int nsite,
	    int nsim, int condl, int iseed, int ismle)
{
  /* IN:
   * treefile positioned at the start of the tree of interest. labels of 
   * internal nodes give BP either as % (0-100) or as proportion (0-1).
   *   Assumes trifurcation at root as in PHYLIP
   *
   * Substitution model:
   * nchar - number of character states
   * smodel - 0:JC, 1:F81, 2:F84, 3:GTR, 4:PAM, 5:JTT, 6:WAG, 7:mtREV24, 8:HKY,
   *          9:LG
   * fr - frequencies; reset for JC
   *      if *fr < 0 for aa models, model-based frequencies used
   * Qk - rate matrix for GTR, ttratio, K or Kappa for F84, HKY, < 0 => ttratio; 
   * not used otherwise. 
   * 
   * Rate distribution: nrate, rate, wt
   * if rate[0] < 0, a discrete gamma model w/ param alpha=-rate[0] is used 
   * calculation of rates same as paml 4.2
   * If tinfo=1 below, a gamma model should be used
   *
   * Info calculation:
   * tinfo - 0:exact, 1:using simulated seq, 2: using sequence seq
   * nsite - only used if tinfo=1
   * seqfile - only used if tinfo=2, contains sequence data
   *
   * nsim - number of simulations used
   * condl - conditional definition of BP or not
   * iseed - used to set seed
   *
   * ismle - if true, adjustment is made for this being mle split
   *
   * RETURN:
   * tstr - string with the tree in Newick format having adjusted BP as labels
   *        adjusted BP in % (0-100)
   */
  int i,is_percentage,iBP,has_names=0,*seq,nsitel;
  char (*names)[11],**ilabels,**alabels,*tstr,*seqc;
  double *utreec,*BP,*aBP;

  nsitel = nsite;
  names = (char (*)[11]) malloc((size_t) ntaxa*sizeof(*names));
  utreec=(double *) malloc((size_t) 4*(ntaxa-1)*sizeof(double));
  BP=(double *) malloc((size_t) (ntaxa-3)*sizeof(double));
  aBP=(double *) malloc((size_t) (ntaxa-3)*sizeof(double));
  alabels = (char **) malloc((size_t) (ntaxa-2)*sizeof(char *));
  if (tinfo==1){
    seq  = (int *) malloc((size_t) ntaxa*nsitel*sizeof(int));
  }

  for(i = 0; i < ntaxa-2; i++) 
    alabels[i]=(char *) malloc((size_t) 4*sizeof(char *));
  
  if(tinfo==2){
    fscanf(seqfile,"%i %i",&i,&nsitel);
    seq  = (int *) malloc((size_t) ntaxa*nsitel*sizeof(int));
    seqc  = (char *) malloc((size_t) ntaxa*nsitel*sizeof(char));
    has_names=1;
    rinterleavef_nolim(seqfile,&ntaxa,&nsitel,names,seqc);
    if (nchar==20)
      for(i = 0; i < ntaxa*nsitel; i++) seq[i]=l2ip(seqc[i]);
    if (nchar==4)
      for(i = 0; i < ntaxa*nsitel; i++) seq[i]=l2i(seqc[i]);
  }
  ilabels=read_treecs(treefile,ntaxa,names,utreec,has_names);
/*   pr_utreec(stdout,ntaxa,utreec); */
/*   exit(0); */
  /* for(i = 0; i < ntaxa-2; i++) printf("%s\n",ilabels[i]); */
  /* for(i = 0; i < ntaxa; i++) printf("%s\n",names[i]); */
  for(i = 0; i < ntaxa-3; i++)
      BP[i] = atof(ilabels[i]);
  is_percentage = 0;
  for(i = 0; i < ntaxa-3; i++)
    if (BP[i] > 1.1){
      is_percentage = 1;
      break;
    }
  if (is_percentage) 
    for(i = 0; i < ntaxa-3; i++) BP[i] /= 100.0;
  
  /* for(i = 0; i < ntaxa-3; i++) printf("%f\n",BP[i]); */
  /*   printf("ntaxa %i nchar %i smodel %i nrate %i nsite %i tinfo %i \n", */
  /*   	 ntaxa, nchar, smodel, nrate, nsitel, tinfo); */
  /* printf("nsim %i condl %i iseed %i\n", nsim, condl, iseed); */
/*   if(!fast_aBP) */
/*     aBPuf(BP,ntaxa,utreec,nchar,smodel,fr,Qk,nrate,rate,wt,tinfo,nsitel,seq,nsim, */
/* 	  condl,iseed,ismle,aBP); */
/*   if(fast_aBP) */
    aBPuf_fast(BP,ntaxa,utreec,nchar,smodel,fr,Qk,nrate,rate,wt,tinfo,nsitel,
	       seq,nsim,condl,iseed,ismle,aBP);
  
  for(i = 0; i < ntaxa-3; i++){
    iBP=100.0*aBP[i];
    sprintf(alabels[i],"%i",iBP);
  }
  alabels[ntaxa-3][0]='\0';
  tstr=ut2in(ntaxa,utreec,names,alabels);

  for(i = 0; i < ntaxa-2; i++){
    free(ilabels[i]); free(alabels[i]);
  }
  free(ilabels); free(alabels);
  free(names); free(utreec); free(BP); 
  if (tinfo==1 || tinfo==2) free(seq); 
  if (tinfo==2) free(seqc); 
  return(tstr);
}
void aBP_paramf(FILE *fp, FILE **bpfile, FILE **utreecfile, FILE **treefile, 
		FILE **frfile, FILE **Qfile, FILE **ratefile, FILE **seqfile, 
		char *aaRmat, int *nBP, int *ntaxa, int *nchar, int *model, 
		int *tinfo, int *ncatG, int *nrate, int *nsim, int *iseed, 
		int *nsite, double *kappa, double *alpha, int *ismle){
  char c,opt[100],value[100];
  int is_option,done=0;
  *bpfile=NULL;
  *utreecfile=NULL;
  *treefile=NULL;
  *frfile=NULL;
  *Qfile=NULL;
  *ratefile=NULL;
  *seqfile=NULL;
  *aaRmat='\0';
  *nchar=4;
  *nrate=1;
  *ncatG=4;
  *alpha=-1;
  *tinfo=0;
  *nsim=10000;
  *nsite=10000;
  *ismle=0;
  *iseed=1;
  while(!done){
    c = ctl_paraml(fp,opt,value,&is_option);
    if(is_option){
/*       printf("option: %s value: %s\n",opt, value); */
      if(strcmp(opt,"ntaxa")==0) *ntaxa=atoi(value);
      if(strcmp(opt,"treefile")==0) 
	*treefile=fopene(value,"r","treefile not available");

      if(strcmp(opt,"nchar")==0) *nchar=atoi(value);
      if(strcmp(opt,"model")==0) *model=atoi(value);
      if(strcmp(opt,"aaRatefile")==0) sprintf(aaRmat,"%s",value);
      if(strcmp(opt,"frfile")==0) 
	*frfile=fopene(value,"r","frfile not available");
      if(strcmp(opt,"Qfile")==0) 
	*Qfile=fopene(value,"r","Qfile not available");
      if(strcmp(opt,"kappa")==0) *kappa=atof(value);
      if(strcmp(opt,"ttratio")==0) *kappa=-atof(value);
      
      if(strcmp(opt,"alpha")==0) *alpha=atof(value);
      if(strcmp(opt,"ncatG")==0) *ncatG=atoi(value);
      if(strcmp(opt,"nrate")==0) *nrate=atoi(value);
      if(strcmp(opt,"ratefile")==0)
	*ratefile=fopene(value,"r","ratefile not available");

      if(strcmp(opt,"tinfo")==0) *tinfo=atoi(value);
      if(strcmp(opt,"nsite")==0) *nsite=(int) atof(value);
      if(strcmp(opt,"seqfile")==0) 
	*seqfile=fopene(value,"r","seqfile not available");

      if(strcmp(opt,"nsim")==0) *nsim=(int) atof(value);
      if(strcmp(opt,"iseed")==0) *iseed=(int) atof(value);

      if(strcmp(opt,"ismle")==0) *ismle=atoi(value);

      if(strcmp(opt,"bpfile")==0) 
	*bpfile=fopene(value,"r","bpfile not available");
      if(strcmp(opt,"nBP")==0) *nBP=atoi(value);
    }
    if(c==EOF) done=1;
  }
}
FILE *fopene(char *filename, char *rw, char *err){
  FILE *fp;
  if((fp=fopen(filename,rw))==NULL){
    printf("%s\n",err);
    exit(1);
  }
  return(fp);
}
char ctl_paraml(FILE *fp, char *opt, char *value, int *is_option){
  /* return last char on line */
  char c;
  int i;

  *is_option=0;
  while(((c = getc(fp)) == ' ' || c == '\t') && c != EOF && c != '\n');
  if(c == EOF || c == '\n') return(c);
  if(c == '*'){ /* comment */
    while((c = getc(fp)) != '\n' && c != EOF) ;
    return(c);
  } 

  /* option name */
  *is_option=1;
  for(i=0; c !=' ' && c != '\t'; i++){
    opt[i]=c;
    c=getc(fp);
  }
  opt[i] = '\0';
  /* skip '=' */
  while((c=getc(fp))==' ' || c=='\t' || c == '=') ;
  for(i=0; c != ' ' && c != '\t' && c != '\n' && c != EOF; i++){
    value[i]=c;
    c=getc(fp);
  }
  value[i]= '\0';
  while(c != '\n' && c != EOF) c=getc(fp);
  return(c);
}
int pmodel2smodel(int model, char *aaRmat, int nchar){
  if (nchar==4){
    if(model==0) return(0); /* JC */
    if(model==2) return(1); /* F81 */
    if(model==3) return(2); /* F84 */
    if(model==4) return(8); /* HKY */
    if(model==7) return(3); /* GTR */
  }
  if (nchar != 4){
    if(model==0) return(0); /* JC */
    if(model==1) return(1); /* F81 */
    if(model==8) return(3); /* GTR */
  }
  if (*aaRmat!='\0'){
    if(strcmp(aaRmat,"dayhoff.dat")==0) return(4); /* PAM */
    if(strcmp(aaRmat,"jones.dat")==0) return(5); /* JTT */
    if(strcmp(aaRmat,"wag.dat")==0) return(6); /* WAG */
    if(strcmp(aaRmat,"mtREV24.dat")==0) return(7); /* mtREV24 */
    if(strcmp(aaRmat,"lg.dat")==0) return(9); /* LG */
  }
  printf("No substitution model available\n");
  exit(1);
}
void forward_probsbz(int nchar, int ntaxa, int s, double *utreec, double *M, 
		     double *f, double *zf){
  int nchar2,i,k,r,l,j;
  nchar2 = nchar*nchar;

  memcpy(zf,f,s*nchar*sizeof(double));
    
  r=utreec[(s-ntaxa)*4]; l=utreec[1+(s-ntaxa)*4];
  for(i = 0; i < nchar; i++) zf[i+s*nchar]=f[i+r*nchar]*f[i+l*nchar];

  for(k = s+1; k < 2*ntaxa-3; k++){
    r=utreec[(k-ntaxa)*4]; l=utreec[1+(k-ntaxa)*4];
    for(i = 0; i < nchar; i++){
      zf[i+k*nchar] = 0;
      for(j = 0; j < nchar; j++) 
	zf[i+k*nchar] += M[i+j*nchar+k*nchar2]*zf[j+l*nchar]*zf[j+r*nchar];
    }}  

  r=utreec[(ntaxa-3)*4]; l=utreec[1+(ntaxa-3)*4];
  for(i = 0; i < nchar; i++) 
    zf[i+(2*ntaxa-3)*nchar] = zf[i+r*nchar]*zf[i+l*nchar];
}
void backward_probsbz(int nchar, int ntaxa, int s, double *utreec, double *M, 
		      double *zf, double *zb){
  /* zb[i+(r-ntaxa)*nchar] gives prob, starting from i, of data "backward" 
   * from r, including t_r */ 
  int nchar2,k,r,l,j,i;
  nchar2 = nchar*nchar;
  r = utreec[(ntaxa-2)*4]; /* root */
  for(j = 0; j < nchar; j++) zb[j+(ntaxa-2)*nchar]=1;
  memcpy(&zb[(ntaxa-3)*nchar],&zf[r*nchar],nchar*sizeof(double));

  for(k = ntaxa-2; k >= 0; k--)
    for(j=0; j<2; j++){
      r = utreec[j+k*4]; l = utreec[1-j+k*4];
      if(r >= ntaxa && r < 2*ntaxa-3 && r!=s)
	backward_probb(nchar,r,&M[r*nchar2],&zf[l*nchar],&zb[k*nchar],
		       &zb[(r-ntaxa)*nchar]);
      if(r==s)
	for(i = 0; i < nchar; i++)
	  zb[i+(r-ntaxa)*nchar] = zf[i+l*nchar]*zb[i+k*nchar];
    }
}
void lik_deriv_calcbz(int nchar, int ntaxa, int s, int *seq, double *fr, 
		      double *utreec, double *Mp, double *f, double *b, 
		      double *lp){
  int nchar2,k,j,c1,c2,r,l,jup,br;

  nchar2 = nchar*nchar;
  for(k = 0; k < ntaxa-1; k++){
    jup = (k < ntaxa-2)?2:1;
    for(j = 0; j < jup; j++){
      r=utreec[j+k*4]; l=utreec[1-j+k*4];
      if(r!=s && r >= ntaxa){
	c1 = utreec[(r-ntaxa)*4]; c2 = utreec[1+(r-ntaxa)*4];
	br = r;
	if(r > s) br--;
	lp[br] = lik_deriv_iedge(nchar,fr,&b[k*nchar],&f[l*nchar],&f[c1*nchar],
				 &f[c2*nchar],&Mp[r*nchar2]);
      }
      if(r < ntaxa)
	lp[r] = lik_deriv_tedge(nchar,seq[r],fr,&b[k*nchar],&f[l*nchar],
				&Mp[r*nchar2]);
    }}
}
void lik_deriv_topz(int nchar, int ntaxa, int s, int p, double *fr, 
		      double *utreec, 
		      double *Mpz, double *f, double *b, double *lp){
  int c1,c2,poc;
  double *fc1,*fc2,*bk,*fl;

  if(s < ntaxa || s >= 2*ntaxa-3){
    printf("lik_deriv_topz: split %i must be terminal\n",s); exit(1);
  }
  c1=utreec[(s-ntaxa)*4]; c2=utreec[1+(s-ntaxa)*4];
  fc1=&f[c1*nchar]; fc2=&f[c2*nchar];
  if(utreec[(p-ntaxa)*4]!=s &&  utreec[1+(p-ntaxa)*4]!=s){
    printf("lik_deriv_topz: wrong parent\n"); exit(1);
  }
  poc=utreec[(p-ntaxa)*4]; /* other child of parent to split of interest */
  if(utreec[1+(p-ntaxa)*4]!=s) poc=utreec[1+(p-ntaxa)*4];
  
  if(p==2*ntaxa-2){ /* root */
    if(poc < ntaxa){
      printf("lik_deriv_topz: problem at root node\n"); exit(1);
    }
    c1=utreec[(poc-ntaxa)*4]; c2=utreec[1+(poc-ntaxa)*4];
    bk=&f[c1*nchar]; fl=&f[c2*nchar];
  }
  else{
    bk=&b[(p-ntaxa)*nchar]; fl=&f[poc*nchar];
  }
    
  lp[0]=lik_deriv_iedge(nchar,fr,bk,fl,fc1,fc2,Mpz);
  lp[1]=lik_deriv_iedge(nchar,fr,bk,fc1,fl,fc2,Mpz);
  lp[2]=lik_deriv_iedge(nchar,fr,bk,fc2,fc1,fl,Mpz);
}
void ederivz(int nchar, int ntaxa, int *seq, double *fr, 
	       double *utreec, double *M, double *Mp, double *Mpz, 
	       int ns, int *s, double *lp, double *px){
  int is,i,r,*p,j,nchar2;
  double *f,*b,*zf,*zb;
  int nb;
  nb=2*ntaxa-3;
  
  nchar2 = nchar*nchar;
  
  f = (double *) malloc((size_t) nchar*(2*ntaxa-2)*sizeof(double));
  b = (double *) malloc((size_t) nchar*(ntaxa-1)*sizeof(double));
  zf = (double *) malloc((size_t) nchar*(2*ntaxa-2)*sizeof(double));
  zb = (double *) malloc((size_t) nchar*(ntaxa-1)*sizeof(double));
  p = (int *) malloc((size_t) (2*ntaxa-2)*sizeof(double));

  for(i = 0; i < ntaxa-1; i++)
    for(j = 0; j < 2; j++) p[(int) utreec[j+i*4]]=i+ntaxa;

  forward_probsb(nchar,ntaxa,seq,utreec,M,f);
  backward_probsb(nchar,ntaxa,utreec,M,f,b);

/*   for(i = 0; i < nb*nchar2; i++) fprintf(fpcheckb,"%.16e\n",M[i]); */
/*   for(i = 0; i < nb*nchar2; i++) fprintf(fpcheckb,"%.16e\n",Mp[i]); */
/*   exit(0); */
  
  for(is = 0; is < ns; is++){
    forward_probsbz(nchar,ntaxa,s[is],utreec,M,f,zf);
    r = utreec[(ntaxa-2)*4]; 
    px[is]=0.0;
    for(i = 0; i < nchar; i++)
      px[is] += fr[i]*zf[i+(2*ntaxa-3)*nchar]*zf[i+r*nchar];
    backward_probsbz(nchar,ntaxa,s[is],utreec,M,zf,zb);
    lik_deriv_calcbz(nchar,ntaxa,s[is],seq,fr,utreec,Mp,zf,zb,
		     &lp[is*(2*ntaxa-1)]);
    lik_deriv_topz(nchar,ntaxa,s[is],p[s[is]],fr,utreec,Mpz,f,b,
		   &lp[2*ntaxa-4+is*(2*ntaxa-1)]);
  }

  free(b); free(f); free(zb); free(zf); free(p);
}
void infomxz(int nchar, int ntaxa, int *seq, double *fr, double *utreec, 
	       int nrate, double *wt, double *M, double *Mp, double *Mpz,
	       int ns, int *s, double *infos, double *px){
  int i,ir,is,llp,nb,nchar2,j;
  double *lp,*lpr,*pxr;
  llp = (2*ntaxa-1);
  nb=2*ntaxa-3;
  nchar2=nchar*nchar;
  lp = (double *) malloc((size_t) llp*ns*sizeof(double));
  lpr = (double *) malloc((size_t) llp*ns*sizeof(double));
  pxr = (double *) malloc((size_t) ns*sizeof(double));

  for(i = 0; i < ns*llp; i++) lp[i] = 0.0;
  for(i = 0; i < ns; i++) px[i] = 0.0;

  for(ir = 0; ir < nrate; ir++){
    ederivz(nchar,ntaxa,seq,fr,utreec,&M[ir*nb*nchar2],&Mp[ir*nb*nchar2],
	    &Mpz[ir*nchar2],ns,s,lpr,pxr);
    for(is = 0; is < ns; is++){
      for(i = 0; i < 2*ntaxa-1; i++) 
	lp[i+is*llp] += wt[ir]*lpr[i+is*llp];
      px[is] += wt[ir]*pxr[is];
    }
  }
  
  for(is = 0; is < ns; is++)
    for(i = 0; i < llp; i++)
      for(j = i; j < llp; j++)
	infos[i+j*llp+is*llp*llp] = (lp[i+is*llp]/px[is])*(lp[j+is*llp]/px[is]);
  free(lp); free(lpr); free(pxr);
}
void transz(int nchar, int ntaxa, double *utreec, 
	    int nrate, double *rate, double *wt,
	    double *fr, double *W, double *Lam,		      
	    double pij(int i, int j, double t, int nchar, 
		       double *fr, double *W, double *Lam),
	    double ppij(int i, int j, double t, int nchar, 
			double *fr, double *W, double *Lam),
	    double *M, double *Mp, double *Mpz){
  int nchar2,nb,i,x,y,j;
  
  nchar2 = nchar*nchar;
  nb = 2*ntaxa-3;
  trans_gen_rate(nchar,ntaxa,utreec,nrate,rate,wt,fr,W,Lam,pij,ppij,M,Mp);
  for(j = 0; j < nrate; j++)
    for(i = 0; i < nb*nchar2; i++) Mp[i+j*nchar2*nb] *= rate[j];
  for(x = 0; x < nchar; x++)
    for(y = 0; y < nchar; y++) Mpz[x+y*nchar]=ppij(x,y,0,nchar,fr,W,Lam);
  for(j = nrate-1; j >= 0; j--)
    for(i = 0; i < nchar2; i++) Mpz[i+j*nchar2] = rate[j]*Mpz[i];
}
void symfillmat(int n, double *a){
  int i,j;
  for(i = 0; i < n; i++)
    for(j = i+1; j < n; j++) a[j+i*n]=a[i+j*n];
}
void infomatrixz(int ntaxa, double *utreec, int ns, int *s,
		 int nchar, int smodel, double *fr, double *Qk, 
		 int nrate, double *rate, double *wt,
		 int tinfo, int nsite, int *seq, double *infom, 
		 int iseed){
  int allocW=0,ix,is,i,dimi,dimi2,*patt,j,nb,npatt,*count;
  int npatt_alloc,is_same,*seqt;
  double *px,*infos,*Mp,*M,*Mpz,*W,*Lam,alpha,*nutreec,pxt;

  double (*pij)(int i, int j, double t, int nchar, double *fr, double *W, 
		double *Lam);
  double (*ppij)(int i, int j, double t, int nchar, double *fr, double *W,
		 double *Lam);
  /* rate distribution */
  alpha=-rate[0];
  if (rate[0] < 0){
    DiscreteGamma(wt,rate,alpha,alpha,nrate,0);
    for(i = 0; i < nrate; i++) wt[i] = 1.0/nrate;
  }

  nb=2*ntaxa-3;
  M=(double *) malloc(nrate*nb*nchar*nchar*sizeof(double));
  Mp=(double *) malloc(nrate*nb*nchar*nchar*sizeof(double));
  Mpz=(double *) malloc(nrate*nchar*nchar*sizeof(double));
  allocW=subst_model_set(nchar,smodel,fr,Qk,&pij,&ppij,&W,&Lam);
  transz(nchar,ntaxa,utreec,nrate,rate,wt,fr,W,Lam,pij,ppij,M,Mp,Mpz);

  if(tinfo==0){
    npatt = 1;
    for(i = 0; i < ntaxa; i++) npatt *= nchar;
    patt = (int *) malloc((size_t) ntaxa*npatt*sizeof(int));
    npatt = all_sitepatt(nchar,ntaxa,patt);
  }
  if(tinfo==2){
    count = (int *) malloc((size_t) nsite*sizeof(int));
    npatt=patt_freq(ntaxa,nsite,seq,count);
    patt=seq;
  }
  if(tinfo==1){
    npatt_alloc=nsite;
    patt=(int *) malloc(npatt_alloc*ntaxa*sizeof(int));
    count=(int *) malloc(npatt_alloc*ns*sizeof(int));
    nutreec=(double *) malloc((ntaxa-1)*4*sizeof(double));
    npatt=0;
    for(is = 0; is < ns; is++){
      memcpy(nutreec,utreec,(ntaxa-1)*4*sizeof(double));
      for(i = 0; i < ntaxa-1; i++)
	for(j = 0; j < 2; j++) 
	  if(nutreec[j+i*4] == s[is]) nutreec[j+2+i*4] = 0;
      seq_genc(smodel,ntaxa,nutreec,nsite,seq,nrate,alpha,iseed,fr,Qk);
      is_same=0;
      for(i = 0; i < nsite; i++){
	for(ix = 0; ix < npatt; ix++){
	  is_same=1;
	  for(j = 0; j < ntaxa; j++)
	    if(seq[i+j*nsite]!=patt[j+ix*ntaxa]){
	      is_same=0;
	      break;
	    }
	  if(is_same){
	    count[is+ix*ns]++;
	    break;
	  }}
	  
	if(!is_same){
	  if(npatt==npatt_alloc){
	    npatt_alloc=npatt+nsite;
	    patt=(int *) realloc(patt,npatt_alloc*ntaxa*sizeof(int));
	    count=(int *) realloc(count,ns*npatt_alloc*sizeof(int));
	  }
	  for(j = 0; j < ntaxa; j++) patt[j+npatt*ntaxa]=seq[i+j*nsite];
	  for(j = 0; j < ns; j++) count[j+npatt*ns]=(j==is)?1:0;
	  npatt++;
	}
      }
    }
  }

  dimi=(2*ntaxa-1);
  dimi2=dimi*dimi;
  px = (double *) malloc(ns*sizeof(double));
  infos = (double *) malloc(ns*dimi2*sizeof(double));
  seqt = (int *) malloc(ntaxa*sizeof(int));

/*   fpcheckb = fopen("infomz.out","w"); */

  for(ix = 0; ix < npatt; ix++){
    if(tinfo==0 || tinfo==2)
      for(i = 0; i < ntaxa; i++) seqt[i] = patt[ix+i*npatt];
    if(tinfo==1)
      for(i = 0; i < ntaxa; i++) seqt[i] = patt[i+ix*ntaxa];
    infomxz(nchar,ntaxa,seqt,fr,utreec,nrate,wt,M,Mp,Mpz,ns,s,infos,px);
    for(is = 0; is < ns; is++){
      if(tinfo==0) pxt=px[is];
      if(tinfo==1) pxt=((double) count[is+ix*ns])/nsite;
      if(tinfo==2) pxt=((double) count[ix])/nsite;
      for(i = 0; i < dimi; i++)
	for(j = i; j < dimi; j++)
	  infom[i+j*dimi+is*dimi2] += pxt*infos[i+j*dimi+is*dimi2];
    }}
  
  for(is = 0; is < ns; is++)
    symfillmat(2*ntaxa-1,&infom[is*dimi2]);
  if(allocW==1 || allocW==2) free(W);
  if(allocW==2) free(Lam);
  free(Mpz); free(M); free(Mp); 
  free(seqt); 
  if(tinfo==0 || tinfo==1)  free(patt); 
  if(tinfo >= 1) free(count); 
  if(tinfo == 1) free(nutreec);
  free(px); free(infos); 
}
int l2i(char c){
  if (c == 'a' || c == 'A') return(0);
  if (c == 'c' || c == 'C') return(1);
  if (c == 'g' || c == 'G') return(2);
  if (c == 't' || c == 'T') return(3);
  if (c == '-') return(5);
  return(6);
}
int l2ip(char c){
  switch(c){
  case 'A':
    return(alanine);
    
  case 'R':
    return(arginine);
    
  case 'N':
    return(asparagine);
    
  case 'D':
    return(aspartic);

  case 'C':
    return(cysteine);

  case 'Q':
    return(glutamine);

  case 'E':
    return(glutamic);

  case 'G':
    return(glycine);

  case 'H':
    return(histidine);

  case 'I':
    return(isoleucine);

  case 'L':
    return(leucine);

  case 'K':
    return(lysine);

  case 'M':
    return(methionine);

  case 'F':
    return(phenylalanine);

  case 'P':
    return(proline);

  case 'S':
    return(serine);

  case 'T':
    return(threonine);

  case 'W':
    return(tryptophan);

  case 'Y':
    return(tyrosine);

  case 'V':
    return(valine);

  case '-':
    return(21);
    
  case 'X':
    return(22);

  case '?':
    return(23);
    /* future: cases X and ? represent unknown aa 
     * B - asparagine or aspartic
     * Z - glutamine or glutamic
     * * - stop codon
     */
  }
  return(100);
}
int rseqf(FILE *fp, char *seq){
  int rsite = 0;
  char c;
  
  while((c = getc(fp)) != '\n' &&  c != EOF){
    if (c != ' ' && c != '\r' && c!= '\t'){
      *seq = c;
      seq++;
      rsite++;
    }
  }

  return(rsite);
}
char ignore_whitespace_seqfile(FILE *infile){
  char c;
  while((c = getc(infile)) == ' ' || c == '\n' || c == '\r' || c == '\t') ;
  if(c == EOF){
    printf("seqfile: EOF before entire sequence read");
    exit(1);
  }
  return(c);
}
int rinterleave_block(FILE *infile, int numsp, int nsite, char name[][11], 
		       char *seq, int *initb){
  char c;
  int i,j,lsite,lsiteo;
  
  for(i = 0; i < numsp; i++){
    c = ignore_whitespace_seqfile(infile);
    if((*initb) == 1){
      name[i][0] = c;
      for(j = 1; j < 10; j++)name[i][j] = getc(infile);
      name[i][10] = '\0';
      c = ignore_whitespace_seqfile(infile);
    }
    seq[i*nsite] = c;
    lsite = rseqf(infile,&seq[1+i*nsite])+1;
    if(i==0) lsiteo=lsite;

    if(i > 0 && lsite != lsiteo){
      printf("seqfile: number of character states should be same for all taxa in a block\n");
      exit(1);
    }
  }
  *initb = 0;
  
  return(lsite);
}
void rinterleavef_nolim(FILE *infile, const int *numsp, const int *nsite, 
			char name[][11], char *seq)
{
  int lsite = 0, csite = 0, initb = 1;

  while(csite < *nsite){
    lsite = rinterleave_block(infile,*numsp,*nsite,name,&seq[csite],&initb);
    csite += lsite;
  }
}
int patt_freq(int numsp, int nsite, int *seq, int *count)
{
  int *s;
  int endsite,i,j,k,match;

  s = (int *) malloc((size_t) nsite*numsp*sizeof(int));
  endsite = 0;
  for(i = 0; i < nsite; i++){
    /* check for a match */
    match = 0;
    for(k = 0; k < endsite; k++){
      match = 1;
      for(j = 0; j < numsp; j++){
	if (seq[i+j*nsite] != s[j+k*numsp]){
	  match = 0;
	  break;
	}
      }
      if (match){
	count[k] += 1;
	break;
      }
    }
    
    /* if not a match, new element */
    if (!match){
      for(j = 0; j < numsp; j++) s[j+endsite*numsp] = seq[i+j*nsite];
      count[endsite] = 1;
      endsite++;
    }
  }
  

  for(j = 0; j < numsp; j++)
    for(i = 0; i < endsite; i++)
      seq[i+j*endsite] = s[j+i*numsp];

  free(s);
  return(endsite);
}
int all_sitepatt(int nchar, int ntaxa, int *patt)
{
  /* IN:
   * patt: nchar^ntaxa x ntaxa matrix
   *       each row gives a pattern
   * RETURN: nchar^ntaxa
   *
   * updates patt as 
   *            patt 0
   *            patt 1
   *            ...
   *            patt nchar
   */
  int npatt,npattf,i,j,b,l;
  
  npattf = 1;
  for(i = 0; i < ntaxa; i++) npattf *= nchar;
  for(b = 0; b < nchar; b++) patt[b] = b;
  npatt = nchar;
  for(i = 1; i < ntaxa; i++){

    for(j = 0; j <= i-1; j++)
      for(b = 0; b < nchar; b++)
	for(l = 0; l < npatt; l++){
	  patt[l+b*npatt+j*npattf] = patt[l+j*npattf];
	}

    for(b = 0; b < nchar; b++)
      for(l = 0; l < npatt; l++){ 
	patt[l+b*npatt+i*npattf] = b;
      }

    npatt *= nchar;
  }

  return(npattf);
}
void makeprotfreqs(double *fr, double *W, double *Lam) 
{
  long i, mineig,nchar=20;
  mineig = 0;
  for (i = 0; i < nchar; i++)
    if (fabs(Lam[i]) < fabs(Lam[mineig]))
      mineig = i;
  for(i = 0; i < nchar; i++) fr[i] =  fabs(W[i+mineig*nchar]);
}
void WLamfr2exchange(int nchar, double *W, double *Lam, double *fr, double *R){
  int i,j,k;
  for(i = 0; i < nchar; i++)
    for(j = (i+1); j < nchar; j++){
      R[i+j*nchar] = 0.0;
      for(k = 0; k < nchar; k++) 
	R[i+j*nchar] += W[i+k*nchar]*W[j+k*nchar]*Lam[k];
    }
  
  for(i = 0; i < nchar; i++) 
    for(j = (i+1); j < nchar; j++) R[i+j*nchar] /= (fr[i]*fr[j]);
  for(i = 0; i < nchar; i++) 
    for(j = 0; j < i; j++) R[i+j*nchar] = R[j+i*nchar];
  for(i = 0; i < nchar; i++)R[i+i*nchar] = 0.0;
}
void chQ_freq(int nbin, double *W, double *Lam, double *fr, double *fro)
{
  /* binning/udistfbf.c 
   *     Q = Pi^(-1) W Lam W' 
   * decomposition used to obtain P(t) as
   *     P(t) = Pi^(-1) W exp(Lam t) W'
   * where W is determined from eigen-decomp
   *     Pi^(1/2) Q Pi^(-1/2) = U' Lam U,   W = (Pi)^(1/2) U'
   * To adjust for frequencies Pi*:
   * obtain Q_ij* = Q_ij pi_j* / pi_j
   *        Q_ii* = - sum_{j neq j} Q_ij*
   *        M_ij* =  sqrt(pi_i*) Q_ij* / sqrt(pi_j*)
   *        U and Lam from M = U' Lam U (eigen-decomp)
   *        W = (Pi*)^(1/2) U' */
  int i,j,l;
  double R[400], Offdiag[400], Rt, mu;

/*   for(i = 0; i < nbin; i++) */
/*     for(j = 0; j < nbin; j++){ */
/*       Rt = 0.0; */
/*       for(l = 0; l < nbin; l++) */
/* 	Rt += W[i+l*nbin]*W[j+l*nbin]*Lam[l]; */
/*       Rt /= fro[i]; */
/*       if (i != j) */
/* 	printf("%f\n", Rt); */
/*     } */
/*   exit(0); */

  for(i = 0; i < nbin; i++){ /* R_ij = Q_ij / (pi_i pi_j) */
    for(j = (i+1); j < nbin; j++){
      Rt = 0.0;
      for(l = 0; l < nbin; l++){
	Rt += W[i+l*nbin]*W[j+l*nbin]*Lam[l];
      }
      Rt /= (fro[i]*fro[j]);
      R[i+j*nbin] = Rt; R[j+i*nbin] = Rt;
    }
  }

  for(i = 0; i < nbin; i++){ /* R_ii = Q_ii* */
    Rt = 0.0;
    for(j = 0; j < nbin; j++){
      if (j != i)
	Rt -= R[i+j*nbin]*fr[j];
    }
    R[i+i*nbin] = Rt/fr[i]; /* will cancel with fr[i] below */
  }

/*   for(i = 0; i < nbin; i++) */
/*     for(j = 0; j < nbin; j++) printf("%f\n",R[i+j*nbin]*fr[j]); exit(0); */

  for(i = 0; i < nbin; i++){ /* R_ij = entry of M_ij* above */
    for(j = i; j < nbin; j++){
      R[i+j*nbin] *= sqrt(fr[i]*fr[j]);
      if (j != i) R[j+i*nbin] = R[i+j*nbin];
    }
  }

  eigenRealSym(R, nbin, Lam, Offdiag); /* eigen-decomp of M* */
  for(i = 0; i < nbin; i++){ /* W = (Pi*)^(1/2) U' */
    for(j = 0; j < nbin; j++){
      W[i+j*nbin] = R[j+i*nbin] * sqrt(fr[i]);
    }
  }

  /* rescale eigenvalues so that -sum_i pi_i Q_ii = 1 */
  mu = 0.0;
  for(i = 0; i < nbin; i++)
    for(l = 0; l < nbin; l++) mu -= Lam[l]*W[i+l*nbin]*W[i+l*nbin];
  for(i = 0; i < nbin; i++) Lam[i] /= mu;
}
double pijJCg(int x, int y, double t, int nchar){
  double m;
  m = nchar/(nchar - 1.0);
  if (x == y) 
    return((1.0/nchar) + exp(-t*m)/m);
  else
    return((1.0/nchar) - (1.0/nchar)*exp(-t*m));
}
double pijJCg_gen(int x, int y, double t, int nchar, double *fr, double *W, 
		  double *Lam){
  return(pijJCg(x,y,t,nchar));
}
double ppijJCg(int x, int y, double t, int nchar){ 
  double m;
  m = nchar/(nchar - 1.0);
  if (x == y) 
    return(-exp(-t*m));
  else
    return(exp(-t*m)/(nchar - 1.0));
}
double ppijJCg_gen(int x, int y, double t, int nchar, double *fr,
		   double *W, double *Lam){ 
  return(ppijJCg(x,y,t,nchar));
}
double pp2ijJCg(int x, int y, double t, int nchar){
  double m;
  m = nchar/(nchar - 1.0);
  if (x == y) 
    return(m*exp(-t*m));
  else
    return(-m*exp(-t*m)/(nchar - 1.0));
}
double muF81f(int nchar, double *fr){
  int l;
  double mu = 0.0;
  for(l = 0; l < nchar; l++) mu += fr[l]*(1.0-fr[l]);
  return(1.0/mu);
}
double pijF81(int i, int j, double t, int nchar, double mu, double *fr){
  if (i == j) return(fr[j]+(1.0-fr[j])*exp(-mu*t));
  if (i != j) return(fr[j]*(1.0-exp(-mu*t)));
  return(fr[j]*(1.0-exp(-mu*t)));
}
double pijF81_gen(int i, int j, double t, int nchar, double *fr, double *mu,
		  double *Lam){
  return(pijF81(i,j,t,nchar,*mu,fr));
}
double ppijF81(int i, int j, double t, int nchar, double mu, double *fr){
  if (i == j) return(-mu*(1.0-fr[j])*exp(-mu*t));
  if (i != j) return(mu*exp(-mu*t)*fr[j]);
  return(mu*exp(-mu*t)*fr[j]);
}
double ppijF81_gen(int i, int j, double t, int nchar, double *fr, double *mu,
		   double *Lam){
  return(ppijF81(i,j,t,nchar,*mu,fr));
}
int is_transitionf(int i, int j){
  return((i==0 && j==2) || (i==2 && j==0) || (i==1 && j==3) || (i==3 && j==1));
}
double muF84f(double K, double *fr){
  double mu = 0.0;
  int l;
/*   printf("%f\n",K); */
  for(l = 0; l < 4; l++) mu += fr[l]*(1.0-fr[l]);
  mu += 2*K*( fr[0]*fr[2]/(fr[0]+fr[2]) + fr[1]*fr[3]/(fr[1]+fr[3]) );
  return(1.0/mu);
}
double pijF84(int i, int j, double t, double mu, double K, double *fr){
  double Pj;
  int is_transition;
/*   printf("1.0/(K+1):%f\n",1.0/(K+1)); */
/*   printf("(K+1)*mu*t:%f\n",(K+1)*mu*t); */
/*   printf("mu*t:%f\n",mu*t); */
  if (j == 0 || j == 2) 
    Pj=fr[0]+fr[2];
  else
    Pj=fr[1]+fr[3];

  if(i == j)
    return(fr[j]+fr[j]*(1/Pj - 1)*exp(-mu*t)+(1-fr[j]/Pj)*exp(-mu*(K+1)*t));

  if (i != j){
    is_transition = is_transitionf(i,j);
    if (is_transition)
      return(fr[j]+fr[j]*(1/Pj - 1)*exp(-mu*t)-(fr[j]/Pj)*exp(-mu*(K+1)*t));
    else
      return(fr[j]*(1.0-exp(-mu*t)));
  }
  return(-1.0);
}
double pijF84_gen(int i, int j, double t, int nchar, double *fr, double *mu,
		  double *K){
  return(pijF84(i,j,t,*mu,*K,fr));
}
double ppijF84(int i, int j, double t, double mu, double K, double *fr){
  double Pj,f;
  int is_transition;
  if (j == 0 || j == 2) 
    Pj=fr[0]+fr[2];
  else
    Pj=fr[1]+fr[3];
  
  if(i == j)
    f=-mu*fr[j]*(1/Pj - 1)*exp(-mu*t) + 
      -1.0*mu*(K+1)*(1-fr[j]/Pj)*exp(-mu*(K+1)*t);
  
  if (i != j){
    is_transition = is_transitionf(i,j);
    if (is_transition)
      f = -mu*fr[j]*(1/Pj - 1)*exp(-mu*t)+
	mu*(K+1)*(fr[j]/Pj)*exp(-mu*(K+1)*t);
    else
      f = mu*fr[j]*exp(-mu*t);
  }
  return(f);
}
double ppijF84_gen(int i, int j, double t, int nchar, double *fr, double *mu,
		  double *K){
  return(ppijF84(i,j,t,*mu,*K,fr));
}
double tt2K(double R, double *fr){
  double pY,pR,K;
  pY=fr[1]+fr[3]; pR=fr[0]+fr[2];
  K=(R*pR*pY-fr[0]*fr[2]-fr[1]*fr[3])/(fr[0]*fr[2]/pR+fr[1]*fr[3]/pY);
  return(K);
}
double K2tt(double K, double *fr){
  double pY,pR,R;
  pY=fr[1]+fr[3]; pR=fr[0]+fr[2];
  R=K*(fr[0]*fr[2]/pR+fr[1]*fr[3]/pY)/(pR*pY) + 
    (fr[0]*fr[2]+fr[1]*fr[3])/(pR*pY);
  return(R);
}
double muHKYf(double kappa, double *fr){
  double mu;
  mu=2.0*(kappa*(fr[0]*fr[2]+fr[1]*fr[3])+(fr[0]+fr[2])*(fr[1]+fr[3]));
  return(1.0/mu);
}
double pijHKY(int i, int j, double t, double mu, double kappa, double *fr){
  int is_transition;
  double A,Pi_j;
  is_transition=is_transitionf(i,j);
  if (i !=j && !is_transition) 
    return(fr[j]*(1.0-exp(-mu*t)));

  if (j == 0 || j == 2) Pi_j=fr[0]+fr[2];
  if (j == 1 || j == 3) Pi_j=fr[1]+fr[3];
  A = 1+Pi_j*(kappa-1);
  if (i == j){
    return(fr[j]+fr[j]*(1/Pi_j - 1)*exp(-mu*t) +(1-fr[j]/Pi_j)*exp(-mu*t*A));
  }
  /* must be transition */
  return(fr[j]+fr[j]*(1/Pi_j - 1)*exp(-mu*t) -fr[j]*exp(-mu*t*A)/Pi_j);
}
double pijHKY_gen(int i, int j, double t, int nchar, double *fr, double *mu,
		  double *kappa){
  return(pijHKY(i,j,t,*mu,*kappa,fr));
}
double ppijHKY(int i, int j, double t, double mu, double kappa, double *fr){
  int is_transition;
  double A,Pi_j;
  is_transition=is_transitionf(i,j);
  if (i !=j && !is_transition) 
    return(mu*fr[j]*exp(-mu*t));

  if (j == 0 || j == 2) Pi_j=fr[0]+fr[2];
  if (j == 1 || j == 3) Pi_j=fr[1]+fr[3];
  A = 1+Pi_j*(kappa-1);
  if (i == j){
    return(-mu*fr[j]*(1/Pi_j-1)*exp(-mu*t)-mu*A*(1-fr[j]/Pi_j)*exp(-mu*t*A));
  }
  /* must be transition */
  return(-mu*fr[j]*(1/Pi_j - 1)*exp(-mu*t)+mu*A*fr[j]*exp(-mu*t*A)/Pi_j);
}
double ppijHKY_gen(int i, int j, double t, int nchar, double *fr, double *mu,
		   double *kappa){
  return(ppijHKY(i,j,t,*mu,*kappa,fr));
}
double tt2kappa(double R, double *fr){
  double pR,pY;
  pR=fr[0]+fr[2];
  pY=fr[1]+fr[3];
  return(R*pR*pY/(fr[0]*fr[2]+fr[1]*fr[3]));
}
double kappa2tt(double kappa,  double *fr){
  double pR,pY;
  pR=fr[0]+fr[2];
  pY=fr[1]+fr[3];
  return(kappa*(fr[0]*fr[2]+fr[1]*fr[3])/(pR*pY));
}
void Q2frWLam(int nbin, double *Q, double *fr, double *W, double *Lam){
  /*     Q = Pi^(-1) W Lam W' 
   *  decomposition used to obtain P(t) as
   *     P(t) = Pi^(-1) W exp(Lam t) W'
   * where W is determined from eigen-decomp
   *     Pi^(1/2) Q Pi^(-1/2) = U' Lam U,   W = (Pi)^(1/2) U' 
   * Assumes Q corresponds to GTR model
   */
  int i,j;
  double R[400], Offdiag[400], mu;

  /* pi_j = [1 + sum_i neq j Q_ji/Q_ij]**(-1) */
  for(j = 0; j < nbin; j++){
    fr[j] = 1.0;
    for(i = 0; i < nbin; i++)
      if (j != i) fr[j] += Q[j+i*nbin]/Q[i+j*nbin];
    fr[j] = 1.0 / fr[j];
  }
  for(i = 0; i < nbin; i++)
    for(j = 0; j < nbin; j++) /* R =  Pi^(1/2) Q Pi^(-1/2) */
      R[i+j*nbin] = sqrt(fr[i])*Q[i+j*nbin]/sqrt(fr[j]);

  /* eigen-decomp of Pi^(1/2) Q Pi^(-1/2) */
  eigenRealSym(R, nbin, Lam, Offdiag); 

  for(i = 0; i < nbin; i++){ /* W = (Pi*)^(1/2) U' */
    for(j = 0; j < nbin; j++)
      W[i+j*nbin] = R[j+i*nbin] * sqrt(fr[i]);
  }

  /* rescale eigenvalues so that -sum_i pi_i Q_ii = 1 */
  mu = 0.0;
  for(i = 0; i < nbin; i++)
    for(j = 0; j < nbin; j++) mu -= Lam[j]*W[i+j*nbin]*W[i+j*nbin];
  for(i = 0; i < nbin; i++) Lam[i] /= mu;
}
double pijGTR(int i, int j, double t, int nchar, double *fr, double *W, 
	      double *Lam){
  int k;
  double f;
  if (t == 0.0){
    if (i == j) return(1.0);
    if (i != j) return(0.0);
  }
  
  f=0.0;
  /* printf("%i\n", nchar); */
  for(k = 0; k < nchar; k++){
    /* printf("%f %f\n", W[i + k*nchar], W[j+ k*nchar]); */
    /* printf("%f\n", Lam[k]); */
    f += W[i + k*nchar] * W[j+ k*nchar] * exp(t*Lam[k]);
  }
  return(f/fr[i]);
}
double ppijGTR(int i, int j, double t, int nchar, double *fr, double *W, 
	       double *Lam){
   int k;
   double f;
   f=0.0;
   for(k = 0; k < nchar; k++) 
     f += Lam[k]*W[i + k*nchar] * W[j+ k*nchar] * exp(t*Lam[k]);
   return(f/fr[i]);
}
int subst_model_set(int nchar, int smodel, double *fr, double *Qk, 
		    double (*(*pij))(int i, int j, double t, int nchar, 
				     double *fr, double *W, double *Lam),
		    double (*(*ppij))(int i, int j, double t, int nchar, 
				      double *fr, double *W, double *Lam),
		    double **W, double **Lam)
{
  /* IN:
   * nchar - number of character states
   * smodel - 0:JC, 1:F81, 2:F84, 3:GTR, 4:PAM, 5:JTT, 6:WAG, 7:mtREV24, 8:HKY,
   *          9:LG
   * Qk - rate matrix for GTR, K or Kappa for F84, HKY; not used otherwise
   *
   * IN/OUT:
   * fr - frequencies; reset for JC
   *      if *fr < 0 for aa models, model-based frequencies used
   * OUT:
   * pij,ppij - transition function and first derivative
   * W,Lam - contain parameters for substitution model
   *
   * RETURN:
   * 2,1 or 0 according to whether both W and Lam were allocated, just W or 
   * neither
   */
  subst_model_setp(smodel,pij,ppij);
  return(subst_model_setparam(nchar,smodel,fr,Qk,W,Lam));
}
int subst_model_setp(int smodel,
		     double (*(*pij))(int i, int j, double t, int nchar,
				     double *fr, double *W, double *Lam),
		     double (*(*ppij))(int i, int j, double t, int nchar,
				       double *fr, double *W, double *Lam)){
  /* IN:
   * smodel - 0:JC, 1:F81, 2:F84, 3:GTR, 4:PAM, 5:JTT, 6:WAG, 7:mtREV24, 8:HKY,
   *          9:LG
   *
   * OUT:
   * pij,ppij - transition function and first derivative
   */
  switch(smodel){
  case 0:
    *pij=&pijJCg_gen; *ppij=&ppijJCg_gen;
    break;
  case 1:
    *pij=&pijF81_gen; *ppij=&ppijF81_gen;
    break;
  case 2:
    *pij=&pijF84_gen; *ppij=&ppijF84_gen;
    break;
  case 3: case 4: case 5: case 6: case 7: case 9:
    *pij=&pijGTR; *ppij=&ppijGTR;
    break;
  case 8:
    *pij=&pijHKY_gen; *ppij=&ppijHKY_gen;
    break;
  }
  return(1);
}
int subst_model_setparam(int nchar, int smodel, double *fr, double *Qk,
			 double **W, double **Lam){
  /* IN:
   * nchar - number of character states
   * smodel - 0:JC, 1:F81, 2:F84, 3:GTR, 4:PAM, 5:JTT, 6:WAG, 7:mtREV24, 8:HKY,
   *          9:LG
   * Qk - rate matrix for GTR, ttratio, K or Kappa for F84, HKY, < 0 => ttratio; 
   * not used otherwise. 
   *
   * IN/OUT:
   * fr - frequencies; reset for JC, determined from Q for GTR
   *      if *fr < 0 for aa models, model-based frequencies used
   * OUT:
   * W,Lam - contain parameters for substitution model
   *
   * RETURN:
   * 2,1 or 0 according to whether both W and Lam were allocated, just W or
   * neither
   */
  int allocWLam=0,aamodel=0,i;
  double *fro,*Wo,*Lamo;
  if (smodel==4 || smodel==5 || smodel==6 || smodel==7 || smodel==9) aamodel=1;

  if (smodel==0){ /* JC */
    for(i = 0; i < nchar; i++) fr[i] = 1.0/nchar;
  }
  if (smodel==1){ /* F81 */
    *W=(double *) malloc((size_t) sizeof(double));
    **W=muF81f(nchar,fr);
/*     for(i = 0; i < nchar; i++) printf("%f\n",fr[i]); printf("\n"); */
/*     printf("%f\n",**W); */
    allocWLam=1;
  }
  if (smodel==2){ /* F84 */
    *W=(double *) malloc((size_t) sizeof(double));
    *Lam=(double *) malloc((size_t) sizeof(double));
/*     printf("%.16e\n",*Qk); */
/*     for(i = 0; i < nchar; i++) printf("%f\n",fr[i]); printf("\n"); */
    **Lam=*Qk;
    if(*Qk < 0) 
      **Lam=tt2K(-(*Qk),fr);
    **W= muF84f(**Lam,fr);
/*     printf("%f %f\n",**W,**Lam); */
    allocWLam=2;
  }
  if (smodel==3){ /* GTR */
    *W = (double *) malloc((size_t) nchar*nchar*sizeof(double));
    *Lam = (double *) malloc((size_t) nchar*sizeof(double));
    Q2frWLam(nchar,Qk,fr,*W,*Lam);
    allocWLam=2;
  }
  if (smodel==4){ /* PAM */
    Wo = pamprobmat; Lamo = pameigmat;
  }
  if (smodel==5){ /* JTT */
    Wo = jttprobmat; Lamo = jtteigmat;
  }
  if (smodel==6){ /* WAG */
    Wo = wagprobmat; Lamo = wageigmat;
  }
  if (smodel==7){ /* mtREV24 */
    Wo = mtREV24probmat; Lamo = mtREV24eigmat;
  }
  if (smodel==8){ /* HKY */
    *W=(double *) malloc((size_t) sizeof(double));
    *Lam=(double *) malloc((size_t) sizeof(double));
    **Lam=*Qk;
    if(*Qk < 0) 
      **Lam=tt2kappa(-(*Qk),fr);
    **W = muHKYf(**Lam,fr);
    allocWLam=2;
  }
  if(smodel==9){ /* LG */
    Wo = lgprobmat; Lamo = lgeigmat;
  }
  if (aamodel){  /* amino acid models */
    if (*fr > 0){ /* different frequencies than for model */
      *W = (double *) malloc((size_t) nchar*nchar*sizeof(double));
      *Lam = (double *) malloc((size_t) nchar*sizeof(double));
      fro = (double *) malloc((size_t) nchar*sizeof(double));
      makeprotfreqs(fro,Wo,Lamo);
      memcpy(*W,Wo,(size_t) nchar*nchar*sizeof(double));
      memcpy(*Lam,Lamo,(size_t) nchar*sizeof(double));
      chQ_freq(nchar,*W,*Lam,fr,fro);
      allocWLam=2;
    }
    else{
      *W=Wo; *Lam=Lamo;
      makeprotfreqs(fr,*W,*Lam);
    }
  }
  return(allocWLam);
}
void trans_gen(int nchar, int ntaxa, double *utreec, double *fr, double *W, 
	       double *Lam, 		      
	       double pij(int i, int j, double t, int nchar, 
			  double *fr, double *W, double *Lam),
	       double ppij(int i, int j, double t, int nchar, 
			     double *fr, double *W, double *Lam),
	       double *M, double *Mp)
{
  /* IN:
   * utreec: the tree, nutreec format, 0 last edge
   * OUT:
   * M, Mp - transition matrices and first derivatives for each of the edges
   */
  int nchar2,join,kup,k,r,x,y;
  double tr;

  nchar2 = nchar*nchar;
  for(join = 0; join < ntaxa-1; join++){
    kup = (join < ntaxa-2)?2:1;
    for(k = 0; k < kup; k++){
      r = (int) utreec[k+join*4];
      tr = utreec[k+2+join*4];
      for(x = 0; x < nchar; x++)
	for(y = 0; y < nchar; y++){
	  M[x+y*nchar+r*nchar2]=pij(x,y,tr,nchar,fr,W,Lam);
	  Mp[x+y*nchar+r*nchar2]=ppij(x,y,tr,nchar,fr,W,Lam);
	}
      /* printf("r: %i, tr: %f\n", r, tr); */
      /* wmat(stdout,nchar,nchar,&M[r*nchar2]); */
/*       wmat(stdout,nchar,nchar,&M[r*nchar2]); */
      /* wmat(stdout,nchar,nchar,&Mp[r*nchar2]); */
    }
  }
}
void trans_gen_rate(int nchar, int ntaxa, double *utreec, 
		    int nrate, double *rate, double *wt,
		    double *fr, double *W, double *Lam, 		      
		    double pij(int i, int j, double t, int nchar, 
			       double *fr, double *W, double *Lam),
		    double ppij(int i, int j, double t, int nchar, 
				double *fr, double *W, double *Lam),
		    double *M, double *Mp)
{
  /* IN:
   * utreec: the tree, nutreec format, 0 last edge
   * OUT:
   * M, Mp - transition matrices and first derivatives for each edges x rate
   *         nchar x nchar x nbranch x nrate 
   *         derivatives are not multiplied by rate
   */
  int nchar2,nb,j,l,i;
  double *utreecn;
  
  nchar2 = nchar*nchar;
  nb = 2*ntaxa-3;
  utreecn = (double *) malloc((size_t) (ntaxa-1)*4*sizeof(double));
  memcpy(utreecn,utreec,(ntaxa-1)*4*sizeof(double));
  for(j = 0; j < nrate; j++){
    for(i = 0; i < ntaxa-1; i++)
      for(l = 0; l < 2; l++) utreecn[l+2+i*4] = rate[j]*utreec[l+2+i*4];
    trans_gen(nchar,ntaxa,utreecn,fr,W,Lam,pij,ppij,&M[j*nchar2*nb],
	      &Mp[j*nchar2*nb]);
  }
  free(utreecn);
  return;
}
void covscorecor(int ntaxa, double *infom, double *Sc)
{
  /* IN:
   * infom: information matrix for site pattern (igorning second deriv part)
   *        format as in infomatrixf3, infomatrixJC
   *        (2 ntaxa - 1) x (2 ntaxa - 1) 
   *      (2 ntaxa - 4) x (2 ntaxa - 4) 11 block gives info entries for 
   *        external edges
   *      (2 ntaxa - 4) x 3 12 (and 21^T) block gives info entries for 
   *        external edges and split of interest
   *      3 x 3 22 block gives info for split of interest
   * OUT:
   * Sc: 3 x 3 matrix giving covariances, corrected for estimation 
   */ 
  int i,j;
  double *Ie,*w,y;

  Ie = (double *) malloc((size_t) (2*ntaxa-4)*(2*ntaxa-4)*sizeof(double));
  w = (double *) malloc((size_t) (2*ntaxa-4)*sizeof(double));

  for(i = 0; i < 2*ntaxa-4; i++)
    for(j = 0; j < 2*ntaxa-4; j++) 
      Ie[i+j*(2*ntaxa-4)]=infom[i+j*(2*ntaxa-1)];
  if (chol(2*ntaxa-4,Ie)){
    printf("External edge information matrix appears singular\n");
    exit(1);
  }
  
  for(i = 0; i < 3; i++){
    memcpy(w,&infom[(2*ntaxa-4+i)*(2*ntaxa-1)],(2*ntaxa-4)*sizeof(double));
    cholf(2*ntaxa-4,Ie,w);
    for(j = i; j < 3; j++){
      y = dot(2*ntaxa-4,&infom[(2*ntaxa-4+j)*(2*ntaxa-1)],1,w,1);
      Sc[i+j*3] = infom[(i+2*ntaxa-4)+(j+2*ntaxa-4)*(2*ntaxa-1)] - y;
      Sc[j+i*3] = Sc[i+j*3];
    }}
  free(Ie); free(w);
}
void corscorecor(int ntaxa, double *infom, double *Sc, double *Ic)
{
  /* IN:
   * infom: information matrix for site pattern (igorning second deriv part)
   *        format as in infomatrixf3, infomatrixJC
   *        (2 ntaxa - 1) x (2 ntaxa - 1) 
   *      (2 ntaxa - 4) x (2 ntaxa - 4) 11 block gives info entries for 
   *        external edges
   *      (2 ntaxa - 4) x 3 12 (and 21^T) block gives info entries for 
   *        external edges and split of interest
   *      3 x 3 22 block gives info for split of interest
   * OUT:
   * Sc: 3 x 3 matrix giving correlations, corrected for estimation 
   * Ic: 3 dimensional giving corrected informations
   */
  int i,j;
  covscorecor(ntaxa,infom,Sc);
  for(j = 0; j < 3; j++) Ic[j] = Sc[j+j*3];
  for(i = 0; i < 3; i++) /* convert to correlation matrix */
    for(j = 0; j < 3; j++) Sc[i+j*3] /= sqrt(Ic[j]*Ic[i]);
}
double BPsf(double *V, double *Sc, int condl){
  /* IN:
   * V - 3d the generated standardized scores for a data set
   * Sc - The correlation matrix of V
   * condl - 1=condl on resolved top estimation, 0=unconditional
   */
  int i,zero=0;
  double d[3],A[3],prop,p,eps=1.0e-9;

  d[0] = V[0]-V[1]; d[1] = V[1]-V[2]; d[2] = V[0]; 
  d[0] /= sqrt(2.0*(1- Sc[0+1*3])); d[1] /= sqrt(2.0*(1-Sc[1+2*3]));
  A[0] = Sc[0+1*3]-Sc[0+2*3]-1+Sc[1+2*3];
  A[0] /= 2*sqrt((1- Sc[0+1*3])*(1- Sc[1+2*3]));
  A[1] = sqrt((1-Sc[0+1*3])/2);
  A[2] = Sc[0+1*3]-Sc[0+2*3];
  A[2] /= sqrt(2*(1-Sc[1+2*3]));
  tvtls_(&zero,d,A,&eps,&prop);
  
  d[0] = V[0]-V[2]; d[1] = V[2]-V[1]; d[2] = V[0]; 
  d[0] /= sqrt(2.0*(1- Sc[0+2*3])); d[1] /= sqrt(2.0*(1-Sc[1+2*3]));
  A[0] = Sc[0+2*3]-Sc[0+1*3]-1+Sc[1+2*3];
  A[0] /= 2*sqrt((1- Sc[0+1*3])*(1- Sc[1+2*3]));
  A[1] = sqrt((1-Sc[0+2*3])/2);
  A[2] = Sc[0+2*3]-Sc[0+1*3];
  A[2] /= sqrt(2*(1-Sc[1+2*3]));
  tvtls_(&zero,d,A,&eps,&p);
  prop += p;

  if (condl){
    for(i = 0; i < 3; i++) d[i] = -V[i];
    A[0] = Sc[0+1*3]; A[1] = Sc[0+2*3]; A[2] = Sc[1+2*3];
    tvtls_(&zero,d,A,&eps,&p);
    prop /= (1.0 - p);
  }

  return(prop);
}
int mle1f(double *V){
  return(V[0] > 0 && V[0] > V[1] && V[0] > V[2]);
}
int bpnf(int nsim, double *rho, double *BP, int condl, int ismle){
  /* IN:
   * rho - 3d gives the correlations, rho_1,2, rho_1,3, rho_2,3
   * ismle - if true adjustment is made for this being mle split
   */
  int i,j,iseq,nsimb=0;
  double *V,Sigma[9],mu[3];

  V = (double  *) malloc((size_t) 3*nsim*sizeof(double));
  for(i = 0; i < 3; i++) Sigma[i+i*3] = 1.0;
  Sigma[0+1*3]=rho[0]; Sigma[0+2*3]=rho[1]; Sigma[1+2*3]=rho[2];
  for(j = 0; j < 3; j++) 
    for(i = j+1; i < 3; i++) Sigma[i+j*3]=Sigma[j+i*3];
  for(i = 0; i < 3; i++) mu[i] = 0.0;
  
  rmvnorm(nsim,3,mu,Sigma,V);
/*   for(iseq = 0; iseq < 100; iseq++) wrow(stdout,3,&V[iseq*3],1); */

  for(iseq = 0; iseq < nsim; iseq++)
    if(!ismle || (ismle && mle1f(&V[iseq*3])))
	BP[nsimb++] = BPsf(&V[iseq*3],Sigma,condl);
  free(V);
  return(nsimb);
}
int eigenRealSym(double A[], int n, double Root[], double Offdiag[])
{
/* This finds the eigen solution of a real symmetrical matrix A[n*n].  In return, 
   A has the right vectors and Root has the eigenvalues. work[n] is the working space.
   The matrix is first reduced to a tridiagonal matrix using HouseholderRealSym(), 
   and then using the QL algorithm with implicit shifts.  

   Adapted from routine tqli in Numerical Recipes in C, with reference to LAPACK
   Ziheng Yang, 23 May 2001
*/
   int status=0;
   HouseholderRealSym(A, n, Root, Offdiag);
   status=EigenTridagQLImplicit(Root, Offdiag, n, A);
   EigenSort(Root, A, n);

   return(status);
}
void EigenSort(double d[], double U[], int n)
{
/* this sorts the eigen values d[] and rearrange the (right) eigen vectors U[]
*/
   int k,j,i;
   double p;

   for (i=0;i<n-1;i++) {
      p=d[k=i];
      for (j=i+1;j<n;j++)
         if (d[j] >= p) p=d[k=j];
      if (k != i) {
         d[k]=d[i];
         d[i]=p;
         for (j=0;j<n;j++) {
            p=U[j*n+i];
            U[j*n+i]=U[j*n+k];
            U[j*n+k]=p;
         }
      }
   }
}
void HouseholderRealSym(double a[], int n, double d[], double e[])
{
/* This uses HouseholderRealSym transformation to reduce a real symmetrical matrix 
   a[n*n] into a tridiagonal matrix represented by d and e.
   d[] is the diagonal (eigends), and e[] the off-diagonal.
*/
   int m,k,j,i;
   double scale,hh,h,g,f;

   for (i=n-1;i>=1;i--) {
      m=i-1;
      h=scale=0;
      if (m > 0) {
         for (k=0;k<=m;k++)
            scale += fabs(a[i*n+k]);
         if (scale == 0)
            e[i]=a[i*n+m];
         else {
            for (k=0;k<=m;k++) {
               a[i*n+k] /= scale;
               h += a[i*n+k]*a[i*n+k];
            }
            f=a[i*n+m];
            g=(f >= 0 ? -sqrt(h) : sqrt(h));
            e[i]=scale*g;
            h -= f*g;
            a[i*n+m]=f-g;
            f=0;
            for (j=0;j<=m;j++) {
               a[j*n+i]=a[i*n+j]/h;
               g=0;
               for (k=0;k<=j;k++)
                  g += a[j*n+k]*a[i*n+k];
               for (k=j+1;k<=m;k++)
                  g += a[k*n+j]*a[i*n+k];
               e[j]=g/h;
               f += e[j]*a[i*n+j];
            }
            hh=f/(h*2);
            for (j=0;j<=m;j++) {
               f=a[i*n+j];
               e[j]=g=e[j]-hh*f;
               for (k=0;k<=j;k++)
                  a[j*n+k] -= (f*e[k]+g*a[i*n+k]);
            }
         }
      } 
      else
         e[i]=a[i*n+m];
      d[i]=h;
   }
   d[0]=e[0]=0;

   /* Get eigenvectors */
   for (i=0;i<n;i++) {
      m=i-1;
      if (d[i]) {
         for (j=0;j<=m;j++) {
            g=0;
            for (k=0;k<=m;k++)
               g += a[i*n+k]*a[k*n+j];
            for (k=0;k<=m;k++)
               a[k*n+j] -= g*a[k*n+i];
         }
      }
      d[i]=a[i*n+i];
      a[i*n+i]=1;
      for (j=0;j<=m;j++) a[j*n+i]=a[i*n+j]=0;
   }
}
int EigenTridagQLImplicit(double d[], double e[], int n, double z[])
{
/* This finds the eigen solution of a tridiagonal matrix represented by d and e.  
   d[] is the diagonal (eigenvalues), e[] is the off-diagonal
   z[n*n]: as input should have the identity matrix to get the eigen solution of the 
   tridiagonal matrix, or the output from HouseholderRealSym() to get the 
   eigen solution to the original real symmetric matrix.
   z[n*n]: has the orthogonal matrix as output

   Adapted from routine tqli in Numerical Recipes in C, with reference to
   LAPACK fortran code.
   Ziheng Yang, May 2001
*/
   int m,j,iter,niter=30, status=0, i,k;
   double s,r,p,g,f,dd,c,b, aa,bb;

   for (i=1;i<n;i++) e[i-1]=e[i];  e[n-1]=0;
   for (j=0;j<n;j++) {
      iter=0;
      do {
         for (m=j;m<n-1;m++) {
            dd=fabs(d[m])+fabs(d[m+1]);
            if (fabs(e[m])+dd == dd) break;  /* ??? */
         }
         if (m != j) {
            if (iter++ == niter) {
               status=-1;
               break;
            }
            g=(d[j+1]-d[j])/(2*e[j]);

            /* r=pythag(g,1); */

            if((aa=fabs(g))>1)  r=aa*sqrt(1+1/(g*g));
            else                r=sqrt(1+g*g);

            g=d[m]-d[j]+e[j]/(g+SIGN(r,g));
            s=c=1;
            p=0;
            for (i=m-1;i>=j;i--) {
               f=s*e[i];
               b=c*e[i];

               /*  r=pythag(f,g);  */
               aa=fabs(f); bb=fabs(g);
               if(aa>bb)       { bb/=aa;  r=aa*sqrt(1+bb*bb); }
               else if(bb==0)             r=0;
               else            { aa/=bb;  r=bb*sqrt(1+aa*aa); }

               e[i+1]=r;
               if (r == 0) {
                  d[i+1] -= p;
                  e[m]=0;
                  break;
               }
               s=f/r;
               c=g/r;
               g=d[i+1]-p;
               r=(d[i]-g)*s+2*c*b;
               d[i+1]=g+(p=s*r);
               g=c*r-b;
               for (k=0;k<n;k++) {
                  f=z[k*n+i+1];
                  z[k*n+i+1]=s*z[k*n+i]+c*f;
                  z[k*n+i]=c*z[k*n+i]-s*f;
               }
            }
            if (r == 0 && i >= j) continue;
            d[j]-=p; e[j]=g; e[m]=0;
         }
      } while (m != j);
   }
   return(status);
}
double dot(int n, double *u, int incu, double *v, int incv)
{
  double f;

  for(f = 0.0; n-- > 0; u += incu, v += incv)
    f += (*u) * (*v);

  return(f);
}
void rnorm(int n, double *v, double *se)    /* Generate n N(0,1)*/
{
  int i;
  double x1, x2;

  for(i=0; i<(n-1); i+=2)
    {x1 = urand(); x2 = urand(); 
    v[i] = sqrt(-2.0*log(x1))*sin(2.0*PI*x2);
    v[i+1] = sqrt(-2.0*log(x1))*cos(2.0*PI*x2);}

  if (n%2 != 0)
    {x1 = urand(); x2 = urand();
    v[n-1] = sqrt(-2.0*log(x1))*sin(2.0*PI*x2);}
}
double urand(void){
  return(drand48());  
/*   return(((double) rand())/(RAND_MAX+1.0)); /\* windows *\/ */
}
int chol(int m, double *l)
{
  int i = 0, ii = 0, j, ij, ik, im;

  for(i = 0, ii = 0, im = 0; i < m; i++, ii += (m+1), im += m){
    for(j = 0, ij = i;  j < i;  j++, ij += m)
      l[ij] = (l[ij] - dot(j, l+i, m, l+j, m))/l[j + j*m];
    for(ik = i; ik < im; ik += m) 
      l[ii] -= l[ik]*l[ik];
    if (l[ii] <= 0.0) 
      return(1);
    l[ii] = sqrt(l[ii]);
  }
  return(0);
}
void cholf(int m, double *l, double *b)
{
  int i, ii;

  for(i = 0; i < m; i++)
    b[i] = (b[i] - dot(i, l+i, m, b, 1))/l[i + i*m];
  for(i = m - 1; i >= 0; i--){
    ii = i + i*m;
    b[i] = (b[i] - dot(m-i-1, l+ii+1, 1, b+i+1, 1))/l[ii];
  }
}
void cholh(int m, double *l, double *b)
{
  int i;
  /* solution of l x = b */
  for(i = 0; i < m; i++)
    b[i] = (b[i] - dot(i, l+i, m, b, 1))/l[i + i*m];
}
int lstring(char * string){
  /* returns length of string */
  int l = 0;
  while (string[l] != '\0') l++;
  return(l);
}
void backward_probb(int nchar, int r, double *M, double *f, double *bk,
		   double *b){
  int i,j;
  for(i = 0; i < nchar; i++){
    b[i] = 0.0;
    for(j = 0; j < nchar; j++) b[i] += M[i+j*nchar]*f[j]*bk[j];
  }
}
void backward_probsb(int nchar, int ntaxa, double *utreec, double *M, double *f, 
		    double *b){
  /* b[i+(r-ntaxa)*nchar] gives prob, starting from i, of data "backward" 
   * from r, including t_r */ 
  int nchar2,k,r,l,j;
  nchar2 = nchar*nchar;
  r = utreec[(ntaxa-2)*4]; /* root */
  for(j = 0; j < nchar; j++) b[j+(ntaxa-2)*nchar]=1;
  memcpy(&b[(ntaxa-3)*nchar],&f[r*nchar],nchar*sizeof(double));

  for(k = ntaxa-2; k >= 0; k--)
    for(j=0; j<2; j++){
      r = utreec[j+k*4]; l = utreec[1-j+k*4];
      if(r >= ntaxa && r < 2*ntaxa-3)
	backward_probb(nchar,r,&M[r*nchar2],&f[l*nchar],&b[k*nchar],
		       &b[(r-ntaxa)*nchar]);
    }
}
void forward_probsb(int nchar, int ntaxa, int *s, double *utreec, double *M, 
		   double *f){
  /* f[i+r*nchar] gives prob, starting from i, of data "forward" from r, 
   * including t_r */ 
  int nchar2,i,k,r,l,j;
  nchar2 = nchar*nchar;

  /* for(k=0; k<2*ntaxa-3; k++){ */
  /*   printf("%i\n",k); */
  /*   wmat(stdout,nchar,nchar,&M[k*nchar2]); */
  /* } */
  /* exit(0); */
      
  
  for(k = 0; k < ntaxa; k++)
    if(s[k] < nchar) 
      memcpy(&f[k*nchar],&M[s[k]*nchar+k*nchar2],nchar*sizeof(double));
    else
      for(i = 0; i < nchar; i++) f[i+k*nchar] = 1.0;

  for(k = ntaxa; k < 2*ntaxa-3; k++){
    r=utreec[(k-ntaxa)*4]; l=utreec[1+(k-ntaxa)*4];
    for(i = 0; i < nchar; i++){
      f[i+k*nchar] = 0;
      for(j = 0; j < nchar; j++) 
	f[i+k*nchar] += M[i+j*nchar+k*nchar2]*f[j+l*nchar]*f[j+r*nchar];
    }}
  
  r=utreec[(ntaxa-3)*4]; l=utreec[1+(ntaxa-3)*4];
  for(i = 0; i < nchar; i++) 
    f[i+(2*ntaxa-3)*nchar] = f[i+r*nchar]*f[i+l*nchar];
}
double lik_deriv_tedge(int nchar, int s, double *fr, double *bk, double *fl, 
		       double *Mp){
  int i;
  double lpr=0.0;

  if(s > nchar) return(0.0);
  for(i = 0; i < nchar; i++) 
    lpr += fr[i]*bk[i]*fl[i]*Mp[i+s*nchar];
  return(lpr);
}
double lik_deriv_iedge(int nchar, double *fr, double *bk, double *fl, 
		     double *fc1, double *fc2, double *Mp){
  int i,j;
  double oi,lpr=0.0;

  for(i = 0; i < nchar; i++){
    oi = 0.0;
    for(j = 0; j < nchar; j++)
      oi += fc1[j]*fc2[j]*Mp[i+j*nchar];
    lpr += fr[i]*bk[i]*fl[i]*oi;
  }
  return(lpr);
}
int sqrtmat(int m, double *A, double *Ah){
  /* returns 0 if eigenvalue had to be set to 0, 1 o/w */
  int i,j,k,izero=0;
  double *R,*Lam,*Offdiag,eps=0.01;
  
  R = (double  *) malloc((size_t) m*m*sizeof(double));
  Offdiag = (double  *) malloc((size_t) m*m*sizeof(double));
  Lam = (double  *) malloc((size_t) m*sizeof(double));

  memcpy(R,A,m*m*sizeof(double));
  eigenRealSym(R,m,Lam,Offdiag);
  for(k = 0; k < m; k++) 
    if (Lam[k] < 0 && Lam[k] > -eps){
      Lam[k]=0.0;
      izero=1;
    }
  for(i = 0; i < m*m; i++) Ah[i] = 0.0;
   for(i = 0; i < m; i++)
    for(j = 0; j < m; j++)
      for(k = 0; k < m; k++) 
	Ah[i+j*m] += R[k+i*m]*sqrt(Lam[k])*R[k+j*m];
   
   free(R); free(Lam); free(Offdiag);
   return(izero);
}
void rmvnorm(int nsim, int m, double *mu, double *Sigma, double *V){
  int i,j,isim;
  double half=0.5,*Z,*Shalf;

  Z = (double  *) malloc((size_t) m*sizeof(double));
  Shalf = (double  *) malloc((size_t) m*m*sizeof(double));
  
  sqrtmat(m,Sigma,Shalf);
  /* wmat(stdout,3,3,Sigma); */
  /* wmat(stdout,3,3,Shalf); */

  for(isim = 0; isim < nsim; isim++){
    rnorm(m,Z,&half);
    for(i = 0; i < m; i++){
      V[i+isim*m] = 0.0;
      for(j = 0; j < m; j++)
      	V[i+isim*m] += Shalf[i+j*m]*Z[j];
      V[i+isim*m] += mu[i];
    }
  }

  free(Z); free(Shalf); 
}
int DiscreteGamma (double freqK[], double rK[], double alpha, double beta, int K, int dgammamean)
{
/* discretization of gamma distribution with equal proportions in each 
   category.
*/
   int i;
   double t, factor=alpha/beta*K, lnga1;

   if (dgammamean==0) {
      lnga1=LnGamma(alpha+1);
      for (i=0; i<K-1; i++) /* cutting points, Eq. 9 */
         freqK[i]=InverseCDFGamma((i+1.0)/K, alpha, beta);
      for (i=0; i<K-1; i++) /* Eq. 10 */
         freqK[i]=IncompleteGamma(freqK[i]*beta, alpha+1, lnga1);

      for(i=0; i<K; i++) rK[i]=freqK[i];

      rK[0] = freqK[0]*factor;
      rK[K-1] = (1-freqK[K-2])*factor;
      for (i=1; i<K-1; i++)  rK[i] = (freqK[i]-freqK[i-1])*factor;
   }
   else {
      for(i=0; i<K; i++) rK[i]=InverseCDFGamma((i*2.+1)/(2.*K), alpha, beta);
      for(i=0,t=0; i<K; i++) t+=rK[i];
      for(i=0; i<K; i++) rK[i]*=factor/t;
   }
   for (i=0; i<K; i++) freqK[i]=1.0/K;

   return (0);
}
void error2 (char * message)
{ printf("\nError: %s.\n", message); exit(-1); }
long factorial (int n)
{
   long f=1, i;
   if (n>10) error2("n>10 in factorial");
   for (i=2; i<=(long)n; i++) f *= i;
   return (f);
}
double LnGamma (double x)
{
/* returns ln(gamma(x)) for x>0, accurate to 10 decimal places.
   Stirling's formula is used for the central polynomial part of the procedure.

   Pike MC & Hill ID (1966) Algorithm 291: Logarithm of the gamma function.
   Communications of the Association for Computing Machinery, 9:684
*/
   double f=0, fneg=0, z, lng;
   int nx=(int)x;

   if((double)nx==x && nx>1 && nx<12)
      lng=log((double)factorial(nx-1));
   else {
      if(x<=0) {
         error2("LnGamma not implemented for x<0");
         if((int)x-x==0) { puts("lnGamma undefined"); return(-1); }
         for (fneg=1; x<0; x++) fneg/=x;
         if(fneg<0) error2("strange!! check lngamma");
         fneg=log(fneg);
      }
      if (x<7) {
         f=1;  z=x-1;
         while (++z<7)  f*=z;
         x=z;   f=-log(f);
      }
      z = 1/(x*x);
      lng = fneg+ f + (x-0.5)*log(x) - x + .918938533204673 
             + (((-.000595238095238*z+.000793650793651)*z-.002777777777778)*z
                  +.083333333333333)/x;
   }
   return  lng;
}
double InverseCDFNormal (double prob)
{
/* returns z so that Prob{x<z}=prob where x ~ N(0,1) and (1e-12)<prob<1-(1e-12)
   returns (-9999) if in error
   Odeh RE & Evans JO (1974) The percentage points of the normal distribution.
   Applied Statistics 22: 96-97 (AS70)

   Newer methods:
     Wichura MJ (1988) Algorithm AS 241: the percentage points of the
       normal distribution.  37: 477-484.
     Beasley JD & Springer SG  (1977).  Algorithm AS 111: the percentage 
       points of the normal distribution.  26: 118-121.
*/
   double a0=-.322232431088, a1=-1, a2=-.342242088547, a3=-.0204231210245;
   double a4=-.453642210148e-4, b0=.0993484626060, b1=.588581570495;
   double b2=.531103462366, b3=.103537752850, b4=.0038560700634;
   double y, z=0, p=prob, p1;

   p1 = (p<0.5 ? p : 1-p);
   if (p1<1e-20) z=999;
   else {
      y = sqrt (log(1/(p1*p1)));   
      z = y + ((((y*a4+a3)*y+a2)*y+a1)*y+a0) / ((((y*b4+b3)*y+b2)*y+b1)*y+b0);
   }
   return (p<0.5 ? -z : z);
}
double InverseCDFChi2 (double prob, double v)
{
/* returns z so that Prob{x<z}=prob where x is Chi2 distributed with df=v
   returns -1 if in error.   0.000002<prob<0.999998
   RATNEST FORTRAN by
       Best DJ & Roberts DE (1975) The percentage points of the 
       Chi2 distribution.  Applied Statistics 24: 385-388.  (AS91)
   Converted into C by Ziheng Yang, Oct. 1993.
*/
   double e=.5e-6, aa=.6931471805, p=prob, g, small=1e-6;
   double xx, c, ch, a=0,q=0,p1=0,p2=0,t=0,x=0,b=0,s1,s2,s3,s4,s5,s6;

   if (p<small)   return(0);
   if (p>1-small) return(9999);
   if (v<=0)      return (-1);

   g = LnGamma (v/2);
   xx=v/2;   c=xx-1;
   if (v >= -1.24*log(p)) goto l1;

   ch=pow((p*xx*exp(g+xx*aa)), 1/xx);
   if (ch-e<0) return (ch);
   goto l4;
l1:
   if (v>.32) goto l3;
   ch=0.4;   a=log(1-p);
l2:
   q=ch;  p1=1+ch*(4.67+ch);  p2=ch*(6.73+ch*(6.66+ch));
   t=-0.5+(4.67+2*ch)/p1 - (6.73+ch*(13.32+3*ch))/p2;
   ch-=(1-exp(a+g+.5*ch+c*aa)*p2/p1)/t;
   if (fabs(q/ch-1)-.01 <= 0) goto l4;
   else                       goto l2;
  
l3: 
   x = InverseCDFNormal(p);
   p1 = 0.222222/v;
   ch = v*pow((x*sqrt(p1)+1-p1), 3.0);
   if (ch>2.2*v+6)
      ch = -2*(log(1-p)-c*log(.5*ch)+g);
l4:
   q=ch;   p1=.5*ch;
   if ((t=IncompleteGamma (p1, xx, g))<0)
      error2 ("\nIncompleteGamma");
   p2=p-t;
   t=p2*exp(xx*aa+g+p1-c*log(ch));   
   b=t/ch;  a=0.5*t-b*c;

   s1=(210+a*(140+a*(105+a*(84+a*(70+60*a))))) / 420;
   s2=(420+a*(735+a*(966+a*(1141+1278*a))))/2520;
   s3=(210+a*(462+a*(707+932*a)))/2520;
   s4=(252+a*(672+1182*a)+c*(294+a*(889+1740*a)))/5040;
   s5=(84+264*a+c*(175+606*a))/2520;
   s6=(120+c*(346+127*c))/5040;
   ch+=t*(1+0.5*t*s1-b*c*(s1-b*(s2-b*(s3-b*(s4-b*(s5-b*s6))))));
   if (fabs(q/ch-1) > e) goto l4;

   return (ch);
}
double IncompleteGamma (double x, double alpha, double ln_gamma_alpha)
{
/* returns the incomplete gamma ratio I(x,alpha) where x is the upper 
           limit of the integration and alpha is the shape parameter.
   returns (-1) if in error
   ln_gamma_alpha = ln(Gamma(alpha)), is almost redundant.
   (1) series expansion,     if (alpha>x || x<=1)
   (2) continued fraction,   otherwise
   RATNEST FORTRAN by
   Bhattacharjee GP (1970) The incomplete gamma integral.  Applied Statistics,
   19: 285-287 (AS32)
*/
   int i;
   double p=alpha, g=ln_gamma_alpha;
   double accurate=1e-10, overflow=1e60;
   double factor, gin=0, rn=0, a=0,b=0,an=0,dif=0, term=0, pn[6];

   if (x==0) return (0);
   if (x<0 || p<=0) return (-1);

   factor=exp(p*log(x)-x-g);   
   if (x>1 && x>=p) goto l30;
   /* (1) series expansion */
   gin=1;  term=1;  rn=p;
 l20:
   rn++;
   term *= x/rn;   gin += term;
   if (term > accurate) goto l20;
   gin *= factor/p;
   goto l50;
 l30:
   /* (2) continued fraction */
   a = 1-p;   b = a+x+1;  term = 0;
   pn[0] = 1;  pn[1] = x;  pn[2] = x+1;  pn[3] = x*b;
   gin = pn[2]/pn[3];
 l32:
   a++;  
   b += 2;
   term++;
   an = a*term;
   for (i=0; i<2; i++) 
      pn[i+4] = b*pn[i+2] - an*pn[i];
   if (pn[5] == 0) goto l35;
   rn = pn[4]/pn[5];
   dif = fabs(gin-rn);
   if (dif > accurate) goto l34;
   if (dif <= accurate*rn) goto l42;
 l34:
   gin = rn;
 l35:
   for (i=0; i<4; i++) pn[i] = pn[i+2];
   if (fabs(pn[4]) < overflow) goto l32;
   for (i=0; i<4; i++) pn[i] /= overflow;
   goto l32;
 l42:
   gin = 1-factor*gin;

 l50:
   return (gin);
}
void parent_branch(int r, int ntaxa, double *utreec, int *p, int *l, 
		   double *bl, double *br)
{
  int i,k,done=0;
  i=r-ntaxa+1; /* only need to check above r */
  if (i < 0) i=0;
  while(!done){
    for(k = 0; k < 2; k++)
      if(((int) utreec[k+i*4])==r){
	*p=i+ntaxa;
	*br=utreec[k+2+i*4];
	*l=utreec[1-k+i*4];
	*bl=utreec[1-k+2+i*4];
	done=1;
      }
    i++;
  }
}
void relab_int(int ntaxa, double *utreec, double *nutreec)
{
  /* assumes r < l */
  int l,la,j,k,r;
 
  for(l = 0; l < 4; l++) nutreec[l+(ntaxa-2)*4] = utreec[l+(ntaxa-2)*4];
  la=2*ntaxa-3;
  /* printf("%i %i\n", (int) nutreec[(ntaxa-2)*4], (int) nutreec[1+(ntaxa-2)*4]); */
  
  for(j=(ntaxa-2); j >= 0; j--)
    for(k = 1; k >= 0; k--){
      r=nutreec[k+j*4];
      if (r >= ntaxa){
	for(l = 0; l < 4; l++) 
	  nutreec[l+(la-ntaxa)*4]=utreec[l+(r-ntaxa)*4];
	nutreec[k+j*4]=la;
	/* printf("j: %i %i\n",(int) nutreec[(ntaxa-2)*4],(int) nutreec[1+(ntaxa-2)*4]); */
	la--;
      }}
}
void rert(int r, int ntaxa, double *utreec, double *nutreec)
{
  /* IN:
   * r - label of branch tree is to be rooted at
   * ntaxa, utreec
   * OUT:
   * nutreec
   *
   * see software notes 2009-06-26
   */
  int p,l,k,kup,j,c,nc;
  double br,bl;
  
  /* If parent node of r is the root, no re-rooting is required. */
  parent_branch(r,ntaxa,utreec,&p,&l,&bl,&br);
  if (p == 2*ntaxa-2){
    for(j = 0; j < 4*(ntaxa-1); j++) nutreec[j]=utreec[j];
    return;
  }

  /* The last row of nutreec will be r 2*ntaxa-3; edge length split in half */
  c=ntaxa-2; 
  nutreec[c*4]=r; nutreec[1+c*4]=2*ntaxa-3;
  nutreec[2+c*4]=br/2; nutreec[3+c*4]=br/2;
  /* printf("%i %i %i\n", c, (int) nutreec[c*4], (int) nutreec[1+c*4]); */

  /* As long as the parent of r is not the root, create new joins in 
   * nutreec that join l with p where p is the parent of l r. 
   * This effectively reverses the orientation of p */
  while(p != 2*ntaxa-2){
    c--; 
    /* The relabeled p will be ntaxa+c-1 rather than l. 
     * New backward branchs start labeling from the root and decrease. */
    nutreec[c*4]=l; nutreec[1+c*4]=ntaxa+c-1;
    nutreec[2+c*4]=bl;
    
    r=p;
    parent_branch(r,ntaxa,utreec,&p,&l,&bl,&br);
    nutreec[3+c*4]=br;
    /* printf("%i %i %i\n", r, p, l); */
    /* printf("%i %i %i\n", c, (int) nutreec[c*4], (int) nutreec[1+c*4]); */
  }
  
  /* finish processing the root join */
  nutreec[1+c*4]=l; nutreec[3+c*4] += bl;
  /* printf("%i %i %i\n\n",  c, (int) nutreec[c*4], (int) nutreec[1+c*4]); */
  nc=c;
  c--;

  /* Start from the root of nutreec and relabel all joins so that they
   * don't conflict with the newly created ones. */
  j=ntaxa-2;
  while(j >= 0){
    kup=(j>nc)?1:2; /* kup set so that newly created joins are not changed */
    for(k = 0; k < kup; k++){
      r=nutreec[k+j*4];
      if (r >= ntaxa){
	/* printf("c j %i %i\n",c,j); */
	for(l = 0; l < 4; l++) 
	  nutreec[l+c*4]=utreec[l+(r-ntaxa)*4]; 
	nutreec[k+j*4]=c+ntaxa;
	c--;
      }
    }
    j--;
  }
}
void rmexspce(int numsp, char names[][11]){
  int i,j;
  for(i = 0; i < numsp; i++)
    for(j = 9; j >= 0; j--)
      if (names[i][j] != ' ' && names[i][j] != '\0' &&  names[i][j] != '\t' && names[i][j] != '\r'){
	names[i][j+1] = '\0';
	break;
      }
}
int ndig_intpart(double f){
  int d=1,ndigits=0;
  while(((int) f/d) > 0){
    d *= 10;
    ndigits++;
  }
  if (ndigits == 0) ndigits=1;
  return(ndigits);
}
char *ut2in(int numsp, double *utreec, char names[][11], char **ilabels){
  char **tstring,*tstr;
  int i,alen,len,r,l;
  double x;

  tstring = (char **) malloc((size_t) (2*numsp-2)*sizeof(char *));

  rmexspce(numsp,names);
  for(i = 0; i < numsp; i++){
    len = lstring(names[i])+1;
    tstring[i] = (char *) malloc((size_t) len*sizeof(char));
    strcpy(tstring[i], names[i]);
  }

  for(i = 0; i < (numsp-3); i++){
    /* memory allocation */
    r = utreec[i*4]; l = utreec[1+i*4];
    len = lstring(tstring[r]) + lstring(tstring[l]) +
      ndig_intpart(utreec[2+i*4]) + ndig_intpart(utreec[3+i*4]) + 18;
    if (r >= numsp) len += lstring(ilabels[r-numsp]);
    if (l >= numsp) len += lstring(ilabels[l-numsp]);
    tstring[i+numsp] = (char *) malloc((size_t) len*sizeof(char));
    /* create subtree */
    if (r >= numsp && l >= numsp)
      alen=sprintf(tstring[i+numsp],"(%s%s:%.5f,%s%s:%.5f)",tstring[r],
	      ilabels[r-numsp],utreec[2+i*4],tstring[l],ilabels[l-numsp],
	      utreec[3+i*4]);
    if (r >= numsp && l < numsp)
      alen=sprintf(tstring[i+numsp],"(%s%s:%.5f,%s:%.5f)",tstring[r],
	      ilabels[r-numsp],utreec[2+i*4],tstring[l],utreec[3+i*4]);
    if (r < numsp && l >= numsp)
      alen=sprintf(tstring[i+numsp],"(%s:%.5f,%s%s:%.5f)",tstring[r],
	     utreec[2+i*4],tstring[l],ilabels[l-numsp],utreec[3+i*4]);
    if (r < numsp && l < numsp)
      alen=sprintf(tstring[i+numsp],"(%s:%.5f,%s:%.5f)",tstring[r],
	      utreec[2+i*4],tstring[l],utreec[3+i*4]);
    /* printf("%i %i: %s\n",alen+1,i+numsp,tstring[i+numsp]); */
  }
  /* PHYLIP requires trifurcation */
  /* memory allocation */
  r = utreec[(numsp-3)*4]; l = utreec[1+(numsp-3)*4];
  len = lstring(tstring[r]) + lstring(tstring[l]) + 16 +
    ndig_intpart(utreec[2+(numsp-3)*4])+ndig_intpart(utreec[3+(numsp-3)*4]);
  if (r >= numsp) len += lstring(ilabels[r-numsp]);
  if (l >= numsp) len += lstring(ilabels[l-numsp]);
  tstring[2*numsp-3] = (char *) malloc((size_t) len*sizeof(char));
  /* create subtree */
  i=numsp-3;
  if (r >= numsp && l >= numsp)
    alen=sprintf(tstring[i+numsp],"%s%s:%.5f,%s%s:%.5f",tstring[r],
	    ilabels[r-numsp],utreec[2+i*4],tstring[l],ilabels[l-numsp],
	    utreec[3+i*4]);
  if (r >= numsp && l < numsp)
    alen=sprintf(tstring[i+numsp],"%s%s:%.5f,%s:%.5f",tstring[r],
	    ilabels[r-numsp],utreec[2+i*4],tstring[l],utreec[3+i*4]);
  if (r < numsp && l >= numsp)
    alen=sprintf(tstring[i+numsp],"%s:%.5f,%s%s:%.5f",tstring[r],
	    utreec[2+i*4],tstring[l],ilabels[l-numsp],utreec[3+i*4]);
  if (r < numsp && l < numsp)
    alen=sprintf(tstring[i+numsp],"%s:%.5f,%s:%.5f",tstring[r],
	    utreec[2+i*4],tstring[l],utreec[3+i*4]);
  /* printf("%i %i %i %i %i: %s\n",r,l,len,alen+1,2*numsp-3,tstring[2*numsp-3]); */
  /* memory allocation for final string */
  r = utreec[(numsp-2)*4]; l = utreec[1+(numsp-2)*4];
  r = (r < l)?r:l;
  x=utreec[2+(numsp-2)*4]+utreec[3+(numsp-2)*4];
  len = lstring(tstring[2*numsp-3]) + lstring(tstring[r]) + 12 + 
    ndig_intpart(x);
  if (r >= numsp) len += lstring(ilabels[r-numsp]);
  tstr=(char *) malloc((size_t) len*sizeof(char));
  /* tree */
  if (r >= numsp)
    alen=sprintf(tstr,"(%s,%s%s:%.5f);",tstring[2*numsp-3],tstring[r],
	    ilabels[r-numsp],x);
  if (r < numsp)
    alen=sprintf(tstr,"(%s,%s:%.5f);",tstring[2*numsp-3],tstring[r],x);
  tstr[len-1] = '\0';
  /* printf("%i %i %i %s\n",r,len,alen+1,tstr); */
  for(i = 0; i < 2*numsp-2; i++) free(tstring[i]);
  free(tstring);
  return(tstr);
}
void ut2n(int numsp, double *utreec, char names[][11], char *tstr){
  char **tstring;
  int i,len,r,l;

  tstring = (char **) malloc((size_t) (2*numsp-2)*sizeof(char *));

  for(i = 0; i < numsp; i++){
    len = lstring(names[i]);
    tstring[i] = (char *) malloc((size_t) len*sizeof(char));
    strcpy(tstring[i], names[i]);
    /* remove trailing blanks */
    while(tstring[i][len-1]==' ') len--;
    tstring[i][len]='\0';
  }

  for(i = 0; i < (numsp-3); i++){
    r = utreec[i*4]; l = utreec[1+i*4];
    len = lstring(tstring[r]) + lstring(tstring[l])+23; /* space for branch lengths in the 100s */
    tstring[i+numsp] = (char *) malloc((size_t) len*sizeof(char));
    sprintf(tstring[i+numsp],"(%s:%.5f,%s:%.5f)",tstring[r],utreec[2+i*4],tstring[l],
	    utreec[3+i*4]);
  }
  /* PHYLIP requires trifurcation */
  r = utreec[(numsp-3)*4]; l = utreec[1+(numsp-3)*4];
  len = lstring(tstring[r]) + lstring(tstring[l]) + 21;
  tstring[2*numsp-3] = (char *) malloc((size_t) len*sizeof(char));
  sprintf(tstring[2*numsp-3],"%s:%.5f,%s:%.5f",tstring[r],utreec[2+(numsp-3)*4],tstring[l],
	    utreec[3+(numsp-3)*4]);
  r = utreec[(numsp-2)*4]; l = utreec[1+(numsp-2)*4];
  r = (r < l)?r:l;
  sprintf(tstr,"(%s,%s:%.5f);",tstring[2*numsp-3],tstring[r],
	    utreec[2+(numsp-2)*4]+utreec[3+(numsp-2)*4]);
  len = lstring(tstr);
  tstr[len] = '\0';
  for(i = 0; i < 2*numsp-2; i++) free(tstring[i]);
  free(tstring);
}
int read_tree(FILE *fp, char *tstring)
{
  /* assumes 
   *   '[' and ']' delimit comments only
   *    '][' won't be used to end a comment and start a new one
   *   white space can be removed */
  int ltstring;
  char c;

  ltstring = 0;
  while((c = getc(fp)) != ';'){ 
    if (c != ' ' && c != '\t' && c != '\r' && c != '\n'){ /* no white space */
      if (c == '[') /* start of comment; ignored */
	while((c = getc(fp)) != ']') ;
      else
	tstring[ltstring++]=c;
    }
  } 
  tstring[ltstring++]=';';
  tstring[ltstring++]='\0';
  return(ltstring);
}
double get_dlist(char *tstr, int ss, int de, int *ssn, int *ls, int *le)
{
  /* IN:
   * tstr - tree
   * ss - char position to start search
   * de - end position of dlist to be searched
   * OUT:
   * ssn - char position to start new search
   * ls - start pos of label; (ls-1) gives end of dlist found
   * le - end pos of label; 
   * RETURN: edge length (0.1 if not present) */

  char c,els[20]; /* note the limit on edge-length string */
  int lb=0,rb=0,lrb,i,j=0;
  double el=0.1;

  i=ss;
  lrb=ss-1;
  while(i <= de-1){
    if ((c=tstr[i]) == '(') lb++;
    if (c == ')'){rb++; lrb=i;}
    if (c == ',' && lb==rb) break;
    i++;
  }
  *ssn=i+1;
  *ls=lrb+1;
  i=lrb+1; 
  while((c=tstr[i]) != ':' && c != ','&& c != ')') i++;
  *le=i-1;
  if(c == ':'){
    while((c=tstr[++i]) != ',' && c != ')') els[j++]=c;
    els[j]='\0';
    el=atof(els);
  }
  return(el);
}
void underscore2blank(char *s){
  while(*s++ != '\0')
    if(*s=='_') *s=' ';
}
char **treecs(int ntaxa, char *tstr, char names[][11], double *utreec, 
	      int has_names){
  /* IN:
   * tstr - Newick tree w/o comments, whitespace and ; (see read_tree)
   * has_names = 1 if names are available
   * IN/OUT:
   * names - if has_names, used to determine ordering 0,...,ntaxa-1 of 
   *         terminal labels
   *         o/w gives names of the taxa labeled 0,...,ntaxa-1 in utreec
   * OUT:
   * utreec 
   * RETURN:
   * ilabels - internal nodel labels; (ntaxa-2); might be uninitialized
   *    not allocated as a contiguous block
   *
   * COMMENTS:
   * Based on observation that a root subtree is either
   *    (a) a leaf and has no `(' or ')' 
   * or (b) a dlist + label and el, and has equal numbers of `(' and ')' 
   *
   * violations of Newick standard
   * - quoted labels not allowed
   * - '[' and ']' only used to delimit comments
   * - limits of 10 non-null chars on leaf names
   * - underscores are not converted to blanks
   */

  int *ds; /* starting char positions of dlists */
  int *de; /* ending char positions of dlists */
  int cd=0,td=1; /* current and total dlists */
  int cj; /* current join */
  int cil; /* current internal label */
  int ctl=0; /* current terminal label */
  int *ul; /* utreec labels */
  int *ls,*le,*is_leaf,k,i,ss,K,j,ssn,ir,il;
  char **ilabels,tname[11];
  double *el; 

  el = (double *) malloc((size_t) ntaxa*sizeof(double));
  ds = (int *) malloc((size_t) 2*ntaxa*sizeof(int));
  de = (int *) malloc((size_t) 2*ntaxa*sizeof(int));
  ls = (int *) malloc((size_t) ntaxa*sizeof(int));
  le = (int *) malloc((size_t) ntaxa*sizeof(int));
  is_leaf = (int *) malloc((size_t) ntaxa*sizeof(int));
  ul = (int *) malloc((size_t) ntaxa*sizeof(int));
  ilabels = (char **)malloc((size_t) (ntaxa-2)*sizeof(char *));

  /* if(has_names) */
  /*   for(i = 0; i < ntaxa; i++) underscore2blank(names[i]); */

  for(i = 0; i < ntaxa; i++) el[i]=0;
  
  *ds=0; *de=0; *is_leaf=0;
  while(tstr[*de] != '\0')(*de)++;
  while(tstr[*de] != ')') (*de)--; /* root label and edge ignored */

  cj=ntaxa-2;
  cil=2*ntaxa-3;
  while (cj >= 0){/* Main loop */

    /* obtain all root subtrees of the current dlist and update dlists */
    ss=ds[cd]+1;
    K=0;
    while(ss <= de[cd]-1){
      el[K]=get_dlist(tstr,ss,de[cd],&ssn,&ls[K],&le[K]);
      is_leaf[K]=(ls[K] == ss)?1:0;
      if (!is_leaf[K]){
	ds[td]=ss;
	de[td]=ls[K]-1;
	td++;
      }
      ss=ssn;
      K++;
    }
/*     printf("current dlist\n"); */
/*     for(j = ds[cd]; j <= de[cd]; j++) putchar(tstr[j]); */
/*     printf("\nnumber of dlists from current: %i\n",td-cd-1); */
/*     for(i = cd+1; i < td; i++){ */
/*       for(j = ds[i]; j <= de[i]; j++) putchar(tstr[j]); */
/*       putchar('\n'); */
/*     } */
/*     printf("number of subtrees: %i\n",K); */
/*     printf("cil: %i cj+ntaxa: %i\n",cil, cj+ntaxa); */

    /* attach ulabels to subtrees */
    for(i = 0; i < K-2; i++){
      ilabels[cil-ntaxa]=(char *)malloc(sizeof(char));
      ilabels[cil-ntaxa][0]='\0';
      cil--;
    }
    for(i=0; i < K; i++){
      if (!is_leaf[i]){
	ilabels[cil-ntaxa]=(char *)malloc((le[i]-ls[i]+2)*sizeof(char));
	memcpy(ilabels[cil-ntaxa],&tstr[ls[i]],(le[i]-ls[i]+1)*sizeof(char));
	ilabels[cil-ntaxa][le[i]+1-ls[i]]='\0';
	/* underscore2blank(ilabels[cil-ntaxa]); */
	ul[i]=cil;
	cil--;
      }
      if (is_leaf[i] && !has_names){
	memcpy(names[ctl],&tstr[ls[i]],(le[i]-ls[i]+1)*sizeof(char));
	for(k=le[i]+1-ls[i]; k < 10; k++) names[ctl][k]=' ';
	names[ctl][10]='\0';
	/* underscore2blank(names[ctl]); */
	ul[i]=ctl;
	ctl++;
      }
      if (is_leaf[i] && has_names){
	memcpy(tname,&tstr[ls[i]],(le[i]-ls[i]+1)*sizeof(char));
	for(k=le[i]+1-ls[i]; k < 10; k++) tname[k]=' ';
	tname[10]='\0';
	/* underscore2blank(tname); */
	for(j = 0; j < ntaxa; j++) 
	  if (strncmp(tname,names[j],11)==0) break;
	ul[i]=j;
      } 
    }/* end of attach ulabels ... */

    /* add root subtrees to utreec */
    for(i = 0; i < K-2; i++){
      utreec[cj*4]=ul[i]; utreec[1+cj*4]=cj-1+ntaxa; 
      utreec[2+cj*4]=el[i]; utreec[3+cj*4]=0.0;
      cj--;
    }
    if (ul[K-2] < ul[K-1]){
      ir=K-2;il=K-1;
    }
    else{
      ir=K-1;il=K-2;
    }
    utreec[cj*4]=ul[ir]; utreec[2+cj*4]=el[ir];
    utreec[1+cj*4]=ul[il]; utreec[3+cj*4]=el[il];
    cj--;
    cd++;

/*     pr_utreec(stdout,ntaxa-(cj+1),&utreec[(cj+1)*4]); */

  } /* end of Main loop */
/*   pr_utreec(stdout,ntaxa,utreec); */
/*   for(i = 0; i < ntaxa; i++) */
/*     printf("%s\n",names[i]); */
  
  free(is_leaf); 
  free(ul); 
  free(le); 
  free(ls); 
  free(de); 
  free(ds); 
  free(el); 
  return(ilabels);
}
char **read_treecs(FILE *treefile, int ntaxa, char names[][11], double *utreec, 
		   int has_names){
  /* IN:
   * treefile - currently reading a new tree
   * IN/OUT:
   * names - if has_names, used to determine ordering 0,...,ntaxa-1 of 
   *         terminal labels
   *         o/w gives names of the taxa labeled 0,...,ntaxa-1 in utreec
   * OUT:
   * utreec 
   * RETURN:
   * ilabels - internal nodel labels; (ntaxa-2); each might only have '\0'
   *    not allocated as a contiguous block
   *
   * COMMENTS:
   * Based on observation that a root subtree is either
   *    (a) a leaf and has no `(' or ')' *
   * or (b) a dlist + label and el, and has equal numbers of `(' and ')' 
   *
   * violations of Newick standard
   * - quoted labels not allowed
   * - '[' and ']' only used to delimit comments
   * - limits of 10 non-null chars on leaf names 
   * - underscores are not converted to blanks
   */
  char tstring[10000];
  char **ilabels;
  read_tree(treefile,tstring);
  /* printf("%s\n",tstring); */
  ilabels=treecs(ntaxa,tstring,names,utreec,has_names);
  return(ilabels);
}
void split_leading1(int numsp, int *split){
  int i;
  if (split[0] == 1) return;
  for(i = 0; i < numsp; i++) split[i] = (split[i] == 1)?0:1;
}
void utreec2splits(int numsp, double *utreec, int *splits){
  int i,r,l,split;
  /* ith column gives split for numsp+ith branch */
  for(split = 0; split < numsp-3; split++){
    r = (int) utreec[split*4]; l = (int) utreec[1+split*4];
    if (r < numsp){
      for(i = 0; i < numsp; i++) splits[i+split*numsp] = 0;
      splits[r+split*numsp] = 1;
    }
    else
      for(i = 0; i < numsp; i++) splits[i+split*numsp] = splits[i+(r-numsp)*numsp];
    if (l < numsp)
      splits[l+split*numsp] = 1;
    else
      for(i = 0; i < numsp; i++) splits[i+split*numsp] += splits[i+(l-numsp)*numsp];
  }
  for(split = 0; split < numsp-3; split++)
    split_leading1(numsp,&splits[split*numsp]);
}
int ivsame(int n, int *x, int *y){
  int i;
  for(i = 0; i < n; i++)
    if (x[i] != y[i]) return(0);
  return(1);
}
